<?php

/* Copyright (C) 2003 Adam Tow

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */


define("kTYPE_USER", 1);
define("kTYPE_GROUP", 2);
define("kACCESS_BAN", 1);
define("kACCESS_ALLOW", 2);
define("kACCESS_ADMINISTRATOR", 1);


/* Function:    RunWithParams
 * 
 * Description: Entry point for soyvice that manages the access rules IP Addresses database table.
 *              This table is referred to when a user logs in to prevent unauthorized login attempts.
 *
 * Parameters:  methodName - the method to call
 *              params - the parameters sent to the script
 *
 * Returns:     Script response XML object
 *
 * Revision History:
 *
 *  2003-03-18 - Initial version (adam)
 *
 *
 */

function RunWithParams($methodName, $params)
{   
    switch($methodName)
    {   
        case "add":
            return Add_Edit_Entry($params);
            break;
        
        case "edit":
            return Generate_Edit_Entry($params);
            break;
        
        case "new":
            return Generate_New_Entry();
            break;
        
        case "delete":
            return Generate_Delete_Confirm($params);
            break;
        
        case "delete_confirm":
            if($params["delete"] && !$params["cancel"]) {
                $ip_address = $params["ip_address"];
                $query = "DELETE FROM soybo_access_ip WHERE ip_address='$ip_address'";
                $result = mysql_query($query);
            }
            break;
        
        default:
            break;
                
    }
    
    $response["template"] = "access.tpl";
    $response["access_list"] = Generate_Access_List();
            
    return ScriptResponse($response);
}



/* Function:    Add_Edit_Entry
 * 
 * Description: Adds an access rule or edits an existing access rule
 *
 * Parameters:  params - the parameters sent to the script
 *
 * Returns:     Script response XML object
 *
 * Revision History:
 *
 *  2003-03-18 - Initial version (adam)
 *
 *
 */

function Add_Edit_Entry($params)
{
    if($params["type"] == "edit" || $params["type"] == "new") {
        
            // Verify the IP Parts
        
        $ip1 = $params["ip1"];
        $ip2 = $params["ip2"];
        $ip3 = $params["ip3"];
        $ip4 = $params["ip4"];
        
        $newAddr = "";
        
            // Construct the IP Address for the rule
        
        if((is_numeric($ip1) && ($ip1 > 0 && $ip1 <= 255)) || $ip1 == "*") {
            if($ip1 == "*")
                $newAddr = "*";
            else {
                $newAddr = $ip1;
                
                if((is_numeric($ip2) && ($ip2 >= 0 && $ip2 <= 255)) || $ip2 == "*") {
                    if($ip2 == "*")
                        $newAddr .= ".*";
                    else {
                        $newAddr .= ".$ip2";
                        
                        if((is_numeric($ip3) && ($ip3 >= 0 && $ip3 <= 255)) || $ip3 == "*") {
                            if($ip3 == "*")
                                $newAddr .= ".*";
                            else {
                                $newAddr .= ".$ip3";
                                
                                if((is_numeric($ip4) && ($ip4 >= 0 && $ip4 <= 255)) || $ip4 == "*") {
                                    $newAddr .= ".$ip4";
                                } else $err = true;
                            }
                        } else $err = true;
                    }
                } else $err = true;
            }   
        } else $err = true;
        
        if($err) {
            $ip_parts = array($ip1, $ip2, $ip3, $ip4);
            $newIP = implode(".", $ip_parts);
            $errStr = "The IP Address you entered \"$newIP\" is invalid. The wildcard * character is acceptable in the following cases: 10.* or 10.0.1.*, but not 10.*.50.1";
        
            return ScriptErrorResponse("", $errStr);
        }

            // Delete the entries for the original IP address and the new IP address. If there's a conflict, sorry! They both get wiped!
                
        $original_ip = $params["ip_address"];

        if($newAddr != $original_ip)
            $query = "DELETE FROM soybo_access_ip WHERE ip_address='$original_ip' OR ip_address='$newAddr'";
        else $query = "DELETE FROM soybo_access_ip WHERE ip_address='$original_ip'";
        $result = mysql_query($query) or die(mysql_error());
        
            // Create the access rules for users and groups
        
        foreach($params as $key => $value) {
            if(($pos = strpos($key, "GROUP_")) !== false) {
                    
                $group_id = str_replace("GROUP_", "", $key);
                $query = "INSERT INTO soybo_access_ip (ip_address, type, user_group_id, access) VALUES ('$newAddr', '2', '$group_id', '$value')";
                $result = mysql_query($query) or die(mysql_error());
                                
            } else if(($pos = strpos($key, "USER_")) !== false) {
            
                    // Add to the database iff the user did not specify to use the group permission for this user
                    
                if($value != 3) {
                    $user_id = str_replace("USER_", "", $key);
                    $query = "INSERT INTO soybo_access_ip (ip_address, type, user_group_id, access) VALUES ('$newAddr', '1', '$user_id', '$value')";
                    $result = mysql_query($query) or die(mysql_error());
                }
            }       
        }

        $response["template"] = "access.tpl";
        $response["sort"] = $sort;
        $response["access_list"] = Generate_Access_List();
        
        return ScriptResponse($response);
    }
}


/* Function:    Generate_New_Entry
 * 
 * Description: Generates the data that will be used in the detail template file
 *
 * Parameters:  None
 *
 * Returns:     Script response object
 *
 * Revision History:
 *
 *  2003-03-18 - Initial version (adam)
 *
 *
 */

function Generate_New_Entry()
{   
    // Get the list of groups
            
    $query = "SELECT * FROM soybo_groups ORDER BY groupname";
    $result = mysql_query($query);
    $groups = array();
    $groupnames = array();
    
    if($result) {
        while($row = mysql_fetch_assoc($result)) {
            $group_id = $row["group_id"];
            $groupnames[$group_id] = $row["groupname"];
            
            if($group_id == kACCESS_ADMINISTRATOR)
                array_push($groups, array("id" => $row["group_id"], "name" => $row["groupname"], "allowed" => true));
            else array_push($groups, array("id" => $row["group_id"], "name" => $row["groupname"], "banned" => true));
        }
    }
    
        // Get a list of users
        
    $query = "SELECT * FROM soybo_users ORDER BY username";
    $result = mysql_query($query);
    $users = array();
    
    if($result) {
        while($row = mysql_fetch_assoc($result)) {
            $user_id = $row["user_id"];
            $groupname = $groupnames[$row["group_id"]];
                    
            array_push($users, array("id" => $row["user_id"], "name" => $row["username"], "group" => $groupname));
        }
    }

    $response["template"] = "detail.tpl";
    $response["groups"] = $groups;
    $response["users"] = $users;
    $response["type"] = "new";
    
    return ScriptResponse($response);
}



/* Function:    Generate_Delete_Confirm
 * 
 * Description: Generates the data that will be used in the delete access rule template file
 *
 * Parameters:  params - array containing the IP Address to search for
 *
 * Returns:     Script response object
 *
 * Revision History:
 *
 *  2003-03-18 - Initial version (adam)
 *
 *
 */

function Generate_Delete_Confirm($params)
{
    global $_DATABASE;
    
    $ip_address = $params["ip_address"];
    
    if($ip_address) {
    
            // Retrieve the entry from the access list table
        
        $query = "SELECT * FROM soybo_access_ip WHERE ip_address='$ip_address'";
        $result = mysql_query($query);
        
        if($result) {
            $num = mysql_num_rows($result);

            if($num > 0) {
                $response["template"] = "delete_confirm.tpl";
                $response["ip_address"] = $ip_address;
            
                return ScriptResponse($response);
            } else {
                return ScriptErrorResponse("", $params["locale"]["errInvalidIPAddress"]);
            }
        } 
    }

    return ScriptErrorResponse("", $params["locale"]["errNoIPAddressSpecified"]);
}



/* Function:    Generate_Edit_Entry
 * 
 * Description: Generates the data that will be used in the detail template file for editing an existing access rule.
 *
 * Parameters:  params - array containing the IP Address to search for
 *
 * Returns:     Script response object
 *
 * Revision History:
 *
 *  2003-03-18 - Initial version (adam)
 *
 *
 */

function Generate_Edit_Entry($params)
{
    $ip_address = $params["ip_address"];
    
    if($ip_address) {
    
            // Retrieve the entry from the access list table
        
        $query = "SELECT * FROM soybo_access_ip WHERE ip_address='$ip_address'";
        $result = mysql_query($query);
        
        if($result) {
            $ip_parts = explode(".", $ip_address);
            
            $bGroups = array();
            $aGroups = array();
            $bUsers = array();
            $aUsers = array();
            
                // Populate the banned and allowed arrays
                
            while($row = mysql_fetch_assoc($result)) {
                $type = $row["type"];
                $access = $row["access"];
                
                if($type == kTYPE_USER) {
                    if($access == kACCESS_BAN) {
                        $bUsers[] = $row["user_group_id"];
                    } else if($access == kACCESS_ALLOW) {
                        $aUsers[] = $row["user_group_id"];
                    }
                } else if($type == kTYPE_GROUP) {
                    if($access == kACCESS_BAN) {
                        $bGroups[] = $row["user_group_id"];
                    } else if($access == kACCESS_ALLOW) {
                        $aGroups[] = $row["user_group_id"];
                    }
                }
            }
            
                // Get the list of groups
            
            $query = "SELECT * FROM soybo_groups ORDER BY groupname";
            $result = mysql_query($query);
            $groups = array();
            $groupnames = array();
            
            if($result) {
                while($row = mysql_fetch_assoc($result)) {
                    $group_id = $row["group_id"];
                    $groupnames[$group_id] = $row["groupname"];
                    
                    if(in_array($group_id, $bGroups))
                        array_push($groups, array("id" => $row["group_id"], "name" => $row["groupname"], "banned" => true));
                    else if(in_array($group_id, $aGroups))
                        array_push($groups, array("id" => $row["group_id"], "name" => $row["groupname"], "allowed" => true));
                    else array_push($groups, array("id" => $row["group_id"], "name" => $row["groupname"]));
                }
            }
            
                // Get a list of users
                
            $query = "SELECT * FROM soybo_users ORDER BY username";
            $result = mysql_query($query);
            $users = array();
            
            if($result) {
                while($row = mysql_fetch_assoc($result)) {
                    $user_id = $row["user_id"];
                    $groupname = $groupnames[$row["group_id"]];
                    
                    if(in_array($user_id, $bUsers))
                        array_push($users, array("id" => $row["user_id"], "name" => $row["username"], "group" => $groupname, "banned" => true));
                    else if(in_array($row["user_id"], $aUsers))
                        array_push($users, array("id" => $row["user_id"], "name" => $row["username"], "group" => $groupname, "allowed" => true));
                    else array_push($users, array("id" => $row["user_id"], "name" => $row["username"], "group" => $groupname, ));
                }
            }
            

            $response["ip1"] = $ip_parts[0];
            $response["ip2"] = $ip_parts[1];
            $response["ip3"] = $ip_parts[2];
            $response["ip4"] = $ip_parts[3];
            
            $response["template"] = "detail.tpl";

            $response["groups"] = $groups;
            $response["users"] = $users;
            
            $response["type"] = "edit";
        
            return ScriptResponse($response);
        } else {
            return ScriptErrorResponse("", "No Access Entry exists with the specified Access ID!");
        }
    }

    return ScriptErrorResponse("", "No Access ID Specified!");
}


/* Function:    Generate_Access_List
 * 
 * Description: Generates the list of access rules for the system.
 *
 * Parameters:  None
 *
 * Returns:     Script response object
 *
 * Revision History:
 *
 *  2003-03-18 - Initial version (adam)
 *
 *
 */

function Generate_Access_List()
{
    global $_DATABASE;
        
    $query = "SELECT * FROM soybo_access_ip ORDER BY ip_address";
    $result = mysql_query($query);
    $numRows = mysql_num_rows($result);
    
    $access_list = array();

    $ip_addresses = array();
    
    for($count = 1; $row = mysql_fetch_assoc($result); ++$count) {
        
        $bannedGroupsStr = false;
        $allowedGroupsStr = false;
        $allowedUsersStr = false;
        $bannedUsersStr = false;
        
            // Check to see if this IP Address has already been processed
        
        $ip = $row["ip_address"];
        
        if(!in_array($ip, $ip_addresses)) {
            $ip_addresses[] = $ip;
            
            // Now do a search in the database for all entries with that IP Address
            
            $query2 = "SELECT * FROM soybo_access_ip WHERE ip_address='$ip'";
            $result2 = mysql_query($query2);
            
            while($row2 = mysql_fetch_assoc($result2)) {
                $access = $row2["access"];

                    // Check to see if this entry is for a user or a group
                
                if($row2["type"] == kTYPE_USER) {
                    
                        // Get the username
                    
                    $username = $_DATABASE->Get_Username($row2["user_group_id"]);
                
                    if($access == kACCESS_BAN) {
                        if(!$username) $username == "???";
                        $bannedUsersStr = (!$bannedUsersStr) ? $username : $bannedUsersStr .= "<br />$username";
                    } else if($access == kACCESS_ALLOW) {
                        if(!$username) $username == "???";
                        $allowedUsersStr = (!$allowedUsersStr) ? $username : $allowedUsersStr .= "<br />$username";
                    }
                
                } else if($row2["type"] == kTYPE_GROUP) {
                
                    $groupname = $_DATABASE->Get_Groupname($row2["user_group_id"]);
                    
                    if($access == kACCESS_BAN) {
                        if(!$groupname) $groupname == "???";
                        $bannedGroupsStr = (!$bannedGroupsStr) ? $groupname : $bannedGroupsStr .= "<br />$groupname";
                    } else if($access == kACCESS_ALLOW) {
                        if(!$groupname) $groupname == "???";
                        $allowedGroupsStr = (!$allowedGroupsStr) ? $groupname : $allowedGroupsStr .= "<br />$groupname";
                    }
                
                    
                } else {
                
                    // This should never happen, the database might be corrupted. We just skip over this entry
                    
                }
            }
            
            $entry = array();
            $entry["ip_address"] = $ip;
            $entry["banned_groups"] = $bannedGroupsStr;
            $entry["allowed_groups"] = $allowedGroupsStr;
            $entry["banned_users"] = $bannedUsersStr;
            $entry["allowed_users"] = $allowedUsersStr;
            
            array_push($access_list, $entry);
        }
    }
    
    return $access_list;
}




?>