<?php

/* Copyright (C) 2003 Adam Tow

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
   

/* Function:	RunWithParams
 * 
 * Description: Entry point for soyvice that manages the services installed and pending on
 *				the computer.
 *
 * Parameters:	methodName - the method to call
 *				params - the parameters sent to the script
 *
 * Returns:		Script response XML object
 *
 * Revision History:
 *
 *	2003-03-20 - Initial version (adam)
 *
 *
 */

function RunWithParams($methodName, $params)
{	
	global $configuration, $current_element, $current_data;
	
	$configuration = false;
	$current_element = false;
	$current_date = false;
	
	switch($methodName)
	{			
			// Folder Actions
		
		case "group_folders":			return Group_Folders_Action($params);
		case "user_folders":			return User_Folders_Action($params);
		case "group_folder_new":		return Group_Folder("new", $params);
		case "user_folder_new":			return User_Folder("new", $params);
		case "user_folder_edit":		return User_Folder("edit", $params);
		case "group_folder_edit":		return Group_Folder("edit", $params);

			// Services Actions
			
		case "services_group":			return Group_Services_Action($params);
		case "services_user":			return User_Services_Action($params);
		case "services_group_add":		return Add_Group_Services($params);
		case "services_user_add":		return Add_User_Services($params);
		case "install":					return Install_Services($params, false);
		case "uninstall":				return Uninstall_Services($params);
		case "detail":					return Service_Detail($params);
		case "update":					return Service_Update($params);
		
			// Display Pages
		
		case "display_install":			return Generate_Installed_Services_List();
		case "display_pending":			return Generate_Pending_Services_List();
		case "display_organize_group":	return Generate_Organize_Group_Page($params);
		case "display_organize_user":	return Generate_Organize_User_Page($params);

		default:						return Generate_Default_Page();
	}
}




/* Function:	New_Group_Folder
 * 
 * Description: This function handles creating a new group folder or editing an existing one.
 *
 * Parameters:	params - the parameters array
 *
 * Returns:		Script response object
 *
 * Revision History:
 *
 *	2003-03-20 - Initial version (adam)
 *
 *
 */

function Group_Folder($type, $params)
{
	global $_DATABASE;

	$group_id = $params["group_id"];
	$groupname = $_DATABASE->Get_Groupname($group_id);
	
	if(!$groupname) return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errGroup"]);
	
	$dirname = addslashes($params["dirname"]);
	
	if(!$dirname) return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errDirname"]);
	
	$len = strlen($dirname);
	if($len < 3 || $len > 32) return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errDirname"]);
	
	$description = addslashes($params["description"]);
	if($description && strlen($description) > 255) return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errDescription"]);
	
	$icon = $params["icon"];
	if($icon && strlen($icon) > 32) return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errIcon"]);
	
		// Make sure the directory name does not already exist
	
	$result = $_DATABASE->Query("SELECT * FROM soybo_directories WHERE type='2' AND user_group_id='$group_id' AND dirname='$dirname'");
	$directory = mysql_fetch_assoc($result);
	
	if($type == "new") {
		if($directory)
			return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errDuplicateDirname"]);

		$result = $_DATABASE->Query("INSERT INTO soybo_directories (type, user_group_id, icon, dirname, description) VALUES ('2', '$group_id', '$icon', '$dirname', '$description')");
		
	} else if($type == "edit") {
		$dir_id = $directory["dir_id"];
		$_DATABASE->Query("UPDATE soybo_directories SET icon='$icon', dirname='$dirname', description='$description' WHERE type='2' AND user_group_id='$group_id' AND dir_id='$dir_id'");
	}
	
	return Generate_Organize_Group_Page($params);	
}






/* Function:	User_Folder
 * 
 * Description: This function handles creating a new user folder or editing an existing one.
 *
 * Parameters:	params - the parameters array
 *
 * Returns:		Script response object
 *
 * Revision History:
 *
 *	2003-03-20 - Initial version (adam)
 *
 *
 */

function User_Folder($type, $params)
{
	global $_DATABASE;

	$user_id = $params["user_id"];
	$username = $_DATABASE->Get_Username($user_id);
	
	if(!$username) return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errUser"]);
	
	$dirname = addslashes($params["dirname"]);
	
	if(!$dirname) return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errDirname"]);
	
	$len = strlen($dirname);
	if($len < 3 || $len > 32) return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errDirname"]);
	
	$description = addslashes($params["description"]);
	if($description && strlen($description) > 255) return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errDescription"]);
	
	$icon = $params["icon"];
	if($icon && strlen($icon) > 32) return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errIcon"]);
	
		// Make sure the directory name does not already exist
	
	$result = $_DATABASE->Query("SELECT * FROM soybo_directories WHERE type='1' AND user_group_id='$user_id' AND dirname='$dirname'");
	$directory = mysql_fetch_assoc($result);
	
	if($type == "new") {
		if($directory)
			return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errDuplicateDirname"]);
		
		$_DATABASE->Query("INSERT INTO soybo_directories (type, user_group_id, icon, dirname, description) VALUES ('1', '$user_id', '$icon', '$dirname', '$description')");
		
	} else if($type == "edit") {
		$dir_id = $directory["dir_id"];
		
		$_DATABASE->Query("UPDATE soybo_directories SET icon='$icon', dirname='$dirname', description='$description' WHERE type='1' AND user_group_id='$user_id' AND dir_id='$dir_id'");
	}
	
	return Generate_Organize_User_Page($params);	
}






/* Function:	Add_Group_Services
 * 
 * Description: Adds the selected services to the group
 *
 * Parameters:	params - the parameters array
 *
 * Returns:		Script response object
 *
 * Revision History:
 *
 *	2003-03-20 - Initial version (adam)
 *
 *
 */

function Add_Group_Services($params)
{
	global $_DATABASE;
	
	$group_id = $params["group_id"];
	$groupname = $_DATABASE->Get_Groupname($group_id);
	
	if(!$groupname) return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errGroup"]);
	
	$dir_id = $params["dir_id"];
	if($dir_id == -1) return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errMenu"]);
	
		// Make sure that the directory exists in the group
	
	$result = $_DATABASE->Query("SELECT * FROM soybo_directories WHERE type='2' AND user_group_id='$group_id' AND dir_id='$dir_id'");
	
	if($dir_id == 0 || $row = mysql_fetch_assoc($result)) {
	
			// Iterate through the selected services and add them to the chosen directory
		
		$service_names = $params["selected"];
		foreach($service_names as $service_name) {
			
				// Make sure that service really exists
			
			$result2 = $_DATABASE->Query("SELECT * FROM soybo_services WHERE service_name='$service_name'");
			
			if(mysql_fetch_row($result2)) {
				$_DATABASE->Query("INSERT INTO soybo_services_users_groups (type, user_group_id, dir_id, service_name) VALUES ('2', '$group_id', '$dir_id', '$service_name')");
				
					// Log the event
				
				GFn_Log_Event(kLOG_ADMIN, "Added $service_name to $groupname group.");
			}
		}
		
	} else {
		return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errGroup"]);
	}
	
		// Rebuild the display
	
	return Generate_Organize_Group_Page($params);
}



/* Function:	Add_User_Services
 * 
 * Description: Adds the selected services to a user
 *
 * Parameters:	params - the parameters array
 *
 * Returns:		Script response object
 *
 * Revision History:
 *
 *	2003-03-20 - Initial version (adam)
 *
 *
 */

function Add_User_Services($params)
{
	global $_DATABASE;
	
	$user_id = $params["user_id"];
	$username = $_DATABASE->Get_Username($user_id);
	
	if(!$username) return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errUser"]);
	
	$dir_id = $params["dir_id"];
	if($dir_id == -1) return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errMenu"]);
	
		// Make sure that the directory exists in the group
	
	$result = $_DATABASE->Query("SELECT * FROM soybo_directories WHERE type='1' AND user_group_id='$user_id' AND dir_id='$dir_id'");
	
	if($dir_id == 0 || $row = mysql_fetch_assoc($result)) {
	
			// Iterate through the selected services and add them to the chosen directory
		
		$service_names = $params["selected"];
		
		if(count($service_names) <= 0) return ScriptErrorResponse("", $params["locale"]["errNothingSelected"]);
		
		foreach($service_names as $service_name) {
			
				// Make sure that service really exists
			
			$result2 = $_DATABASE->Query("SELECT * FROM soybo_services WHERE service_name='$service_name'");
			
			if(mysql_fetch_row($result2)) {
				$_DATABASE->Query("INSERT INTO soybo_services_users_groups (type, user_group_id, dir_id, service_name) VALUES ('1', '$user_id', '$dir_id', '$service_name')");
				
					// Log the event
				
				GFn_Log_Event(kLOG_ADMIN, "Added $service_name to $username.");
			}
		}
		
	} else {
		return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errUser"]);
	}
	
		// Rebuild the display
	
	return Generate_Organize_User_Page($params);
}




/* Function:	User_Folders_Action
 * 
 * Description: Actions on directories of a user
 *
 * Parameters:	params - the parameters array
 *
 * Returns:		Script response object
 *
 * Revision History:
 *
 *	2003-03-21 - Initial version (adam)
 *
 *
 */

function User_Folders_Action($params)
{
	global $_DATABASE;

	$user_id = $params["user_id"];
	$username = $_DATABASE->Get_Username($user_id);

	if(!$username) return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errUser"]);
	
	$action = $params["action"];

	if($action == "newfolder") {
		$response["icons"] = Generate_Icon_List();
		$response["username"] = $username;
		$response["user_id"] = $user_id;
		$response["type"] = "user";
		$response["form"] = "new";
		$response["template"] = "folder_detail.tpl";
		
		return ScriptResponse($response);
		
	} else if($action == "edit") {
		$dir_id = $params["dir_id"];
		$result = $_DATABASE->Query("SELECT * FROM soybo_directories WHERE TYPE='1' AND user_group_id='$user_id' AND dir_id='$dir_id'");
		
		$directory = mysql_fetch_assoc($result);
		$response["username"] = $username;
		$response["user_id"] = $user_id;
		$response["directory"] = $directory;
		$response["icons"] = Generate_Icon_List();
		$response["type"] = "user";
		$response["form"] = "edit";
		$response["template"] = "folder_detail.tpl";
		
		return ScriptResponse($response);
		
	} else if($action == "deletefolder" && $params["confirm"] && $params["cancel"]) {
	
		return Generate_Organize_User_Page($params);
		
	} else if($action == "deletefolder" && $params["confirm"]) {
	
				// Iterate through the selected directories
		
		$dir_ids = $params["selected"];
		
		foreach($dir_ids as $dir_id) {

				// Make sure that directory really exists
			
			$result2 = $_DATABASE->Query("SELECT * FROM soybo_directories WHERE type='1' AND user_group_id='$user_id' AND dir_id='$dir_id'");
			
			if($dir_id != 0 && mysql_fetch_row($result2)) {
				
					// Remove services for the directory and the directory itself from the database
					
				$_DATABASE->Query("DELETE FROM soybo_services_users_groups WHERE type='1' AND user_group_id='$user_id' AND dir_id='$dir_id'");
				$_DATABASE->Query("DELETE FROM soybo_directories WHERE type='1' AND user_group_id='$user_id' AND dir_id='$dir_id'");
				
					// Log the events
				
				GFn_Log_Event(kLOG_ADMIN, "Removed services from $dirname directory of $username.");
				GFn_Log_Event(kLOG_ADMIN, "Removed $dirname directory from $username.");	
			}
		}
	}  else if($action == "deletefolder") {
		$response["user_id"] = $user_id;
		$response["selected"] = $params["selected"];
		
		if(count($params["selected"]) > 0) {
			$response["type"] = "user";
			$response["template"] = "confirm_delete_folders.tpl";
				
			return ScriptResponse($response);
		} else {
			return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errNothingSelected"]);
		}
	}

		// Rebuild the display
	
	return Generate_Organize_User_Page($params);
}



/* Function:	Group_Folders_Action
 * 
 * Description: Actions on directories of a group
 *
 * Parameters:	params - the parameters array
 *
 * Returns:		Script response object
 *
 * Revision History:
 *
 *	2003-03-20 - Initial version (adam)
 *
 *
 */

function Group_Folders_Action($params)
{
	global $_DATABASE;

	$group_id = $params["group_id"];
	$groupname = $_DATABASE->Get_Groupname($group_id);

	if(!$groupname) return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errGroup"]);
	
	$action = $params["action"];

	if($action == "newfolder") {
		$response["icons"] = Generate_Icon_List();
		$response["groupname"] = $groupname;
		$response["group_id"] = $group_id;
		$response["type"] = "group";
		$response["form"] = "new";
		$response["template"] = "folder_detail.tpl";
		
		return ScriptResponse($response);
		
	} else if($action == "edit") {
		$dir_id = $params["dir_id"];
		$result = $_DATABASE->Query("SELECT * FROM soybo_directories WHERE TYPE='2' AND user_group_id='$group_id' AND dir_id='$dir_id'");
		
		$directory = mysql_fetch_assoc($result);
		$response["groupname"] = $groupname;
		$response["group_id"] = $group_id;
		$response["directory"] = $directory;
		$response["icons"] = Generate_Icon_List();
		$response["type"] = "group";
		$response["form"] = "edit";
		$response["template"] = "folder_detail.tpl";
		
		return ScriptResponse($response);
		
	} else if($action == "deletefolder" && $params["confirm"] && $params["cancel"]) {
	
		return Generate_Organize_Group_Page($params);
		
	} else if($action == "deletefolder" && $params["confirm"]) {
	
				// Iterate through the selected directories
		
		$dir_ids = $params["selected"];
		
		foreach($dir_ids as $dir_id) {

				// Make sure that directory really exists
			
			$result2 = $_DATABASE->Query("SELECT * FROM soybo_directories WHERE type='2' AND user_group_id='$group_id' AND dir_id='$dir_id'");
			
			if($dir_id != 0 && mysql_fetch_row($result2)) {
				
					// Remove services for the directory and the directory itself
					
				$_DATABASE->Query("DELETE FROM soybo_services_users_groups WHERE type='2' AND user_group_id='$group_id' AND dir_id='$dir_id'");
				$_DATABASE->Query("DELETE FROM soybo_directories WHERE type='2' AND user_group_id='$group_id' AND dir_id='$dir_id'");
				
					// Log the events
				
				GFn_Log_Event(kLOG_ADMIN, "Removed services from $dirname directory of $groupname group");
				GFn_Log_Event(kLOG_ADMIN, "Removed $dirname directory from $groupname group.");	
			}
		}
	}  else if($action == "deletefolder") {
		$response["group_id"] = $group_id;
		$response["selected"] = $params["selected"];
		
		if(count($params["selected"]) > 0) {
			$response["type"] = "group";
			$response["template"] = "confirm_delete_folders.tpl";
				
			return ScriptResponse($response);
		} else {
			return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errNothingSelected"]);
		}
	}

		// Rebuild the display
	
	return Generate_Organize_Group_Page($params);
}



/* Function:	User_Services_Action
 * 
 * Description: Actions on services contained by a user
 *
 * Parameters:	params - the parameters array
 *
 * Returns:		Script response object
 *
 * Revision History:
 *
 *	2003-03-20 - Initial version (adam)
 *
 *
 */

function User_Services_Action($params)
{
	global $_DATABASE;
	
	$user_id = $params["user_id"];
	$username = $_DATABASE->Get_Username($user_id);

	if(!$username) return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errUser"]);
	
	$dir_id = $params["dir_id"];
	if($dir_id == -2) return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errMenu"]);
	
		// Make sure that the directory exists in the group
	
	$result = $_DATABASE->Query("SELECT * FROM soybo_directories WHERE type='1' AND user_group_id='$user_id' AND dir_id='$dir_id'");
	
	if($dir_id == 0 || $dir_id == -1 || $row = mysql_fetch_assoc($result)) {
	
			// Iterate through the selected services
		
		$service_names = $params["selected"];
		foreach($service_names as $service_name) {
			
				// Make sure that service really exists

			$result2 = $_DATABASE->Query("SELECT * FROM soybo_services WHERE service_name='$service_name'");
			
			if(mysql_fetch_row($result2)) {
				if($dir_id == -1) {
					$_DATABASE->Query("DELETE FROM soybo_services_users_groups WHERE type='1' AND user_group_id='$user_id' AND service_name='$service_name'");
					
					GFn_Log_Event(kLOG_ADMIN, "Removed $service_name from $groupname group.");	
				} else {
					$_DATABASE->Query("UPDATE soybo_services_users_groups SET dir_id='$dir_id' WHERE type='1' AND user_group_id='$user_id' AND service_name='$service_name'");
				
						// Log the event
				
					GFn_Log_Event(kLOG_ADMIN, "Moved $service_name within $username.");	
				}
			}
		}
		
	} else {
		return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errUser"]);
	}
	
		// Rebuild the display
	
	return Generate_Organize_User_Page($params);
}






/* Function:	Group_Services_Action
 * 
 * Description: Actions on services contained within a group
 *
 * Parameters:	params - the parameters array
 *
 * Returns:		Script response object
 *
 * Revision History:
 *
 *	2003-03-20 - Initial version (adam)
 *
 *
 */

function Group_Services_Action($params)
{
	global $_DATABASE;
	
	$group_id = $params["group_id"];
	$groupname = $_DATABASE->Get_Groupname($group_id);

	if(!$groupname) return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errGroup"]);
	
	$dir_id = $params["dir_id"];
	if($dir_id == -2) return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errMenu"]);
	
		// Make sure that the directory exists in the group
	
	$result = $_DATABASE->Query("SELECT * FROM soybo_directories WHERE type='2' AND user_group_id='$group_id' AND dir_id='$dir_id'");
	
	if($dir_id == 0 || $dir_id == -1 || $row = mysql_fetch_assoc($result)) {
	
			// Iterate through the selected services
		
		$service_names = $params["selected"];
		foreach($service_names as $service_name) {
			
				// Make sure that service really exists
			
			$result2 = $_DATABASE->Query("SELECT * FROM soybo_services WHERE service_name='$service_name'");
			
			if(mysql_fetch_row($result2)) {
				if($dir_id == -1) {
					$_DATABASE->Query("DELETE FROM soybo_services_users_groups WHERE type='2' AND user_group_id='$group_id' AND service_name='$service_name'");
					
					GFn_Log_Event(kLOG_ADMIN, "Removed $service_name from $groupname group.");	
				} else {
					$_DATABASE->Query("UPDATE soybo_services_users_groups SET dir_id='$dir_id' WHERE type='2' AND user_group_id='$group_id' AND service_name='$service_name'");
				
						// Log the event
				
					GFn_Log_Event(kLOG_ADMIN, "Moved $service_name within $groupname group.");	
				}
			}
		}
		
	} else {
		return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errGroup"]);
	}
	
		// Rebuild the display
	
	return Generate_Organize_Group_Page($params);
}




/* Function:	Generate_Services_List
 * 
 * Description: Generates the list of currenty installed services
 *
 * Parameters:	None
 *
 * Returns:		array of installed services
 *
 * Revision History:
 *
 *	2003-03-18 - Initial version (adam)
 *
 *
 */

function Generate_Services_List()
{
	global $_CONF;
	global $_DATABASE;
	
		// Get a list of installed services
	
	$result = $_DATABASE->Query("SELECT * FROM soybo_services ORDER BY name");
	$numRows = mysql_num_rows($result);
	
	$services = array();
	
	while($row = mysql_fetch_assoc($result)) {
		$services[$row["service_name"]] = $row;
	}
	
	return $services;
}



/* Function:	Uninstall_Services
 * 
 * Description: Uninstalls the selected services from the system. We remove all references to 
 *				the service from all groups and users that contain the service.
 *
 * Parameters:	params - array containing the file paths of the services to install
 *
 * Returns:		true
 *
 * Revision History:
 *
 *	2003-03-20 - Initial version (adam)
 *
 *
 */
 
function Uninstall_Services($params)
{
	global $_DATABASE;
	
	$selected = $params["selected"];
	
	if(!$params["confirm"]) {
		$response["selected"] = $selected;
		
		if(count($params["selected"]) > 0) {
			$response["template"] = "confirm_uninstall_services.tpl";
				
			return ScriptResponse($response);
		} else {
			return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errNothingSelected"]);
		}	
	} else if(!$params["cancel"]) {
	
			// Delete references to the service from all groups and users
			// We do not delete the Service Management service for obvious reasons
			
		foreach($selected as $service_name) {			
			if($service_name != "services_soybo") {
				$_DATABASE->Query("DELETE FROM soybo_services_users_groups WHERE service_name='$service_name'");
				$_DATABASE->Query("DELETE FROM soybo_services WHERE service_name='$service_name'");
			}
		}
	}
	
	return Generate_Installed_Services_List();
}



/* Function:	Generate_Installed_Services_List
 * 
 * Description: Creates the data that will be used by the Installed Services page.
 *
 * Parameters:	None
 *
 * Returns:		Script response object
 *
 * Revision History:
 *
 *	2003-03-20 - Initial version (adam)
 *
 *
 */
 
function Generate_Installed_Services_List()
{
	$services = Generate_Services_List();
	$response["serviceCount"] = count($services);
	$response["services"] = $services;
	$response["template"] = "install.tpl";
		
	return ScriptResponse($response);
}



/* Function:	Generate_Pending_Services_List
 * 
 * Description: Creates the data that will be used by the Pending Services page.
 *
 * Parameters:	None
 *
 * Returns:		Script response object
 *
 * Revision History:
 *
 *	2003-03-20 - Initial version (adam)
 *
 *
 */
 
function Generate_Pending_Services_List()
{
	$services = Generate_Services_List();
	$pending = Generate_Pending_List($services);
	$response["pendingCount"] = count($pending);
	$response["pending"] = $pending;
	$response["template"] = "pending.tpl";
	
	return ScriptResponse($response);
}



/* Function:	Service_Update
 * 
 * Description: Refreshes the service's information in the database by reading
 *				the configuration file from the service folder.
 *
 * Parameters:	params - array containing the the service to update
 *
 * Returns:		Script Response Object
 *
 * Revision History:
 *
 *	2003-03-20 - Initial version (adam)
 *
 *
 */

function Service_Update($params)
{
	global $_CONF;
	global $_DATABASE;
	
	$service_name = $params["service_name"];
	$service = $_DATABASE->Get_Service($service_name);
	
	if(!$service)
		return ScriptErrorResponse("", $params["locale"]["errServiceNotFound"]);
		
	$params["pending"] = array($_CONF["path_services"] . $service["path"] . "/soybo.xml");
	
	Install_Services($params, true);
	
	return Service_Detail($params);
}



/* Function:	Service_Detail
 * 
 * Description: Displays detailed information about a service
 *
 * Parameters:	params - array containing the the service to display
 *
 * Returns:		Script Response Object
 *
 * Revision History:
 *
 *	2003-03-20 - Initial version (adam)
 *
 *
 */

function Service_Detail($params)
{
	global $_CONF;
	global $_DATABASE;
	
	$service_name = $params["service_name"];
	$service = $_DATABASE->Get_Service($service_name);
	
	if(!$service)
		return ScriptErrorResponse("", $params["locale"]["errServiceNotFound"]);
	
	$service["path"] = $_CONF["path_services"] . $service["path"];
	$service["script"] = $service["path"] . "/" . $service["script"];
	$service["icon"] = $service["path"] . "/" . $service["icon"];
	
	$response["template"] = "service_detail.tpl";
	$response["service"] = $service;

	return ScriptResponse($response);	
}






/* Function:	Install_Services
 * 
 * Description: Installs the selected services from the pending folder. We add
 *				the services to the soybo_services database table.
 *
 * Parameters:	params - array containing the file paths of the services to install
 *
 * Returns:		Script Response Object
 *
 * Revision History:
 *
 *	2003-03-20 - Initial version (adam)
 *
 *
 */

function Install_Services($params, $update)
{
	global $_CONF;
	global $_DATABASE;
	global $configuration, $current_element, $current_data, $array_key;

	$paths = $params["pending"];
	
	if(count($paths) <= 0) return ScriptErrorResponse("", $params["locale"]["errNothingSelected"]);
	
	foreach($paths as $filename) {
		if(file_exists($filename)) {
			$xml_parser = xml_parser_create();
			xml_parser_set_option($xml_parser, XML_OPTION_CASE_FOLDING, 0);
			xml_set_element_handler($xml_parser, "_PrvServiceConfStartElement", "_PrvServiceConfEndElement");
			xml_set_character_data_handler($xml_parser, "_PrvServiceConfCharacterData");
		
			if(!($fp = fopen($filename, "r"))) die("could not open XML input: $filename");
			
					// Parse the configuration XML document
			while ($data = fread($fp, 4096)) {
				if (!xml_parse($xml_parser, $data, feof($fp))) {
					die(sprintf("XML error: %s at line %d",
								xml_error_string(xml_get_error_code($xml_parser)),
								xml_get_current_line_number($xml_parser)));
				}
			}
				// Free the parser and close file pointer
				
			xml_parser_free($xml_parser);
			fclose($fp);
			
				// Verify that the configuration file is properly formatted
				
			$err = Verify_Configuration($configuration, $params["locale"]);
			if($err) return ScriptErrorResponse($params["locale"]["errTitle"], $err);
			
				// Extract the information from the configuration array
			
			$service_name = ereg_replace("[^[:alnum:]_-]", "", $configuration["service_name"]);
			$name = addslashes($configuration["name"]);
			$version = $configuration["version"];
			$description = addslashes($configuration["description"]);
			$default_locale = $configuration["default_locale"];
			$script = $configuration["script"];
			$wap_compatible = ($configuration["wap_compatible"] == 1) ? 1 : 0;
			$icon = $configuration["icon"];
			$author = addslashes($configuration["author"]);
			$email = $configuration["email"];
			$website = $configuration["website"];
				
				// Create the path
			
			$path = str_replace("soybo.xml", "", $filename);
			$parts = pathinfo($path);
			$path = $parts["basename"];
				
				// Make sure that the service doesn't already exist in the database
				// If it does, update the entry
				
			$result = $_DATABASE->Query("SELECT * FROM soybo_services WHERE service_name='$service_name'");
			
			if(mysql_num_rows($result) == 0) {
				$query = "INSERT INTO soybo_services
							(service_name, version, name, description, default_locale, script, wap_compatible, icon, author, email, website, path)
							VALUES ('$service_name', '$version', '$name', '$description', '$default_locale', '$script', '$wap_compatible', '$icon', '$author', '$email', '$website', '$path')";
				
				$result = $_DATABASE->Query($query);
			} else if($update) {
				$query = "UPDATE soybo_services SET
							name='$name', version='$version', description='$description', default_locale='$default_locale', script='$script', wap_compatible='$wap_compatible',
							icon='$icon', author='$author', email='$email', website='$website', path='$path' WHERE service_name='$service_name'";
				
				$result = $_DATABASE->Query($query);				
			}
		}
	}

	$response["template"] = "success_install.tpl";

	return ScriptResponse($response);
}



/* Function:	Verify_Configuration
 * 
 * Description: Verifies the information stored in a soybo.xml configuration file.
 *
 * Parameters:	configuration - the parsed array object from a configuration file
 *				locale - the locale object for the service
 *
 * Returns:		false or error
 *
 * Revision History:
 *
 *	2003-03-20 - Initial version (adam)
 *
 *
 */
 
function Verify_Configuration($configuration, $locale)
{
	if(!$configuration["service_name"] || strlen($configuration["service_name"]) > 32)
		return $params["locale"]["errServiceName"] . ": " . $configuration["service_name"];
	
	if(ereg_replace("[^[:alnum:]_-]", "", $configuration["service_name"]) != $configuration["service_name"])
		return $params["locale"]["errServiceNameFormat"];
	
	if(!$configuration["name"] || strlen($configuration["name"]) > 32)
		return $params["locale"]["errServiceTitle"] . ": " . $configuration["name"];
	
	if(!$configuration["script"] || strlen($configuration["script"]) > 32)
		return $params["locale"]["errServiceScript"] . ": " . $configuration["script"];
	
	if($configuration["version"] && strlen($configuration["version"]) > 12)
		return $params["locale"]["errServiceVersion"];
	
	if($configuration["description"] && strlen($configuration["description"]) > 255)
		return $params["locale"]["errServiceDescription"];
	
	if($configuration["default_locale"] && strlen($configuration["default_locale"]) > 33)
		return $params["locale"]["errServiceLocale"];
	
	if($configuration["icon"] && strlen($configuration["icon"]) > 33)
		return $params["locale"]["errServiceIcon"];
	
	if($configuration["author"] && strlen($configuration["author"]) > 100)
		return $params["locale"]["errServiceAuthor"];
	
	if($configuration["website"] && strlen($configuration["website"]) > 100)
		return $params["locale"]["errServiceAuthorWebsite"];
		
	if($configuration["email"] && strlen($configuration["email"]) > 100)
		return $params["locale"]["errServiceAuthorEmail"];
	
	return false;
}


/* Function:	Generate_Icon_List
 * 
 * Description: Generates the list of icons available to the system
 *
 * Parameters:	None
 *
 * Returns:		Script response object
 *
 * Revision History:
 *
 *	2003-03-20 - Initial version (adam)
 *
 *
 */

function Generate_Icon_List()
{
	global $_CONF;
	
	$icons = array();
	
	$path_icons = $_CONF["path_html"] . "/skins/icons/";
	
	if(is_dir($path_icons)) {
		$dir = dir($path_icons);

		while(false !== ($file = $dir->read())) {
	        $filepath = $path_icons . $file;
    	    $filetype = filetype($filepath);

    	    if(substr($file, 0, 1) != "." && $filetype == "file") {
				$parts = pathinfo($filepath);
				$extension = $parts["extension"];
				
				if($extension = "gif" || $extension == "jpg" || $extension == "GIF" || $extension == "JPG") {
					$icons[] = $file;
				}
    	    }  
		}
		$dir->close();
	}
	
	return $icons;
}



/* Function:	Generate_Default_Page
 * 
 * Description: Generates the contents for the default page
 *
 * Parameters:	None
 *
 * Returns:		Script response object
 *
 * Revision History:
 *
 *	2003-03-20 - Initial version (adam)
 *
 *
 */

function Generate_Default_Page()
{
	global $_DATABASE;
	
		// Get a list of groups
	
	$result = $_DATABASE->Query("SELECT * FROM soybo_groups ORDER BY groupname");
	$groups = array();
	
	while($row = mysql_fetch_assoc($result)) {
		$groups[] = $row;	
	}
	
		// Get a list of users
		
	$result = $_DATABASE->Query("SELECT * FROM soybo_users ORDER BY username");
	$users = array();
	
	while($row = mysql_fetch_assoc($result)) {
	
			// Exclude the guest user
		
		if($row["username"] != "guest")
			$users[] = $row;
	}
		
	
	$response["groups"] = $groups;
	$response["users"] = $users;
	$response["template"] = "main.tpl";
	
	return ScriptResponse($response);
}



/* Function:	Generate_Organize_Group_Page
 * 
 * Description: Generates the contents for the organize page for a given group.
 *				We get a list of the group's folders, services, and the set of
 *				services that are available to add.
 *
 * Parameters:	params - the parameter array
 *
 * Returns:		Script response object
 *
 * Revision History:
 *
 *	2003-03-20 - Initial version (adam)
 *
 *
 */

function Generate_Organize_Group_Page($params)
{
	global $_DATABASE;
	global $_CONF;
	
	$group_id = $params["group_id"];
	$groupname = $_DATABASE->Get_Groupname($group_id);
	
	if(!$groupname) return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errGroup"]);
	
		// We don't handle subdirectories at the moment
	
	$dir_id = ($params["dir_id"]) ? $params["dir_id"] : 0;
	$dir_id = 0;
		
		// Get a list of all the services contained by the group
	
	$result = $_DATABASE->Query("SELECT * FROM soybo_services_users_groups WHERE type='2' AND user_group_id='$group_id' ORDER BY dir_id");
	$groupServices = array();	

	while($row = mysql_fetch_assoc($result)) {
		$service_name = $row["service_name"];
		$result2 = $_DATABASE->Query("SELECT * FROM soybo_services WHERE service_name='$service_name'");
		
		$service = mysql_fetch_assoc($result2);
		$service_dir_id = $row["dir_id"];
		$dirname = $_DATABASE->Get_Dirname($service_dir_id);
		$service["dirname"] = $dirname;
		$groupServices[$service_name] = $service;
	}
	ksort($groupServices);
	
		// Get a list of all remaining services
	
	$result = $_DATABASE->Query("SELECT * FROM soybo_services ORDER BY name");
	$numRows = mysql_num_rows($result);
	$remainingServices = array();
	
	while($row = mysql_fetch_assoc($result)) {
		$service_name = $row["service_name"];
		
		if(!array_key_exists($service_name, $groupServices)) {
			$remainingServices[$service_name] = $row;
		}
	}
	
		// Get a list of all the sub-directories for the home directory
		// We add unlimited sub-folders of sub-folders at a later point
	
	$result = $_DATABASE->Query("SELECT * FROM soybo_directories WHERE type='2' AND user_group_id='$group_id' AND dir_parent='0'");
	$directories = array();
		
	while($directory = mysql_fetch_assoc($result)) {
		$dirname = $directory["dirname"];
		$directories[$dirname] = $directory;
	}
	ksort($directories);
	
	if($dir_id == 0) {
		$strings = $_CONF["locale"]->GetStrings();
		$directory = array("dir_id" => 0, "dirname" => $strings["labelHome"], "icon" => "home.gif", "description" => $strings["labelHomeDescription"]);
	} else {
		$result = $_DATABASE->Query("SELECT * from soybo_directories WHERE dir_id='$dir_id'");
		$directory = mysql_fetch_assoc($result);		
	}
	
	$response["template"] = "organize.tpl";
	$response["type"] = "group";
	$response["group_id"] = $group_id;
	$response["groupname"] = $groupname;
	$response["services"] = $groupServices;
	$response["services_remaining"] = $remainingServices;
	$response["subdirectories"] = $directories;
	$response["directory"] = $directory;

	//echo_object(0, $response, "response");
	
	return ScriptResponse($response);
}



/* Function:	Generate_Organize_User_Page
 * 
 * Description: Generates the contents for the organize page for a given user.
 *				We retrieve a list of the user's folders, services, the user's group services
 *				and a list of services that are available to add to the user's account.
 *
 * Parameters:	params - the parameter array
 *
 * Returns:		Script response object
 *
 * Revision History:
 *
 *	2003-03-20 - Initial version (adam)
 *
 *
 */

function Generate_Organize_User_Page($params)
{
	global $_DATABASE;
	global $_CONF;
	
	$user_id = $params["user_id"];
	$username = $_DATABASE->Get_Username($user_id);
	
	if(!$username) return ScriptErrorResponse($params["locale"]["errTitle"], $params["locale"]["errUser"]);
	
		// We don't handle subdirectories at the moment
	
	$dir_id = ($params["dir_id"]) ? $params["dir_id"] : 0;
	$dir_id = 0;
		
		// Get a list of all the services contained by the user
	
	$result = $_DATABASE->Query("SELECT * FROM soybo_services_users_groups WHERE type='1' AND user_group_id='$user_id' ORDER BY dir_id");
	$userServices = array();	

	while($row = mysql_fetch_assoc($result)) {
		$service_name = $row["service_name"];
		$result2 = $_DATABASE->Query("SELECT * FROM soybo_services WHERE service_name='$service_name'");
		
		$service = mysql_fetch_assoc($result2);
		$service_dir_id = $row["dir_id"];
		$dirname = $_DATABASE->Get_Dirname($service_dir_id);
		$service["dirname"] = $dirname;
		$userServices[$service_name] = $service;
	}
	
		// Get a list of all the services contained by the user's group. We will exclude them from the list
	
	$group_id = $_DATABASE->Get_Group_ID_From_User_ID($user_id);
	$groupname = $_DATABASE->Get_Groupname($group_id);
	$result = $_DATABASE->Query("SELECT * FROM soybo_services_users_groups WHERE type='2' AND user_group_id='$group_id' ORDER BY dir_id");
	$groupServices = array();
	
	while($row = mysql_fetch_assoc($result)) {
		$service_name = $row["service_name"];
		$result2 = $_DATABASE->Query("SELECT * FROM soybo_services WHERE service_name='$service_name'");
		
		$service = mysql_fetch_assoc($result2);
		$service_dir_id = $row["dir_id"];
		$dirname = $_DATABASE->Get_Dirname($service_dir_id);
		$service["dirname"] = $dirname;
		$service["groupname"] = $groupname;
		$groupServices[$service_name] = $service;
	}
	ksort($groupServices);
	
	
		// Get a list of all remaining services
	
	$result = $_DATABASE->Query("SELECT * FROM soybo_services ORDER BY name");
	$numRows = mysql_num_rows($result);
	$remainingServices = array();
	
	while($row = mysql_fetch_assoc($result)) {
		$service_name = $row["service_name"];
		
		if(!array_key_exists($service_name, $userServices) && !array_key_exists($service_name, $groupServices)) {
			$remainingServices[$service_name] = $row;
		}
	}
	
		// Get a list of all the sub-directories for the home directory
		// We add unlimited sub-folders of sub-folders at a later point
	
	$result = $_DATABASE->Query("SELECT * FROM soybo_directories WHERE type='1' AND user_group_id='$user_id' AND dir_parent='0'");
	$directories = array();
		
	while($directory = mysql_fetch_assoc($result)) {
		$dirname = $directory["dirname"];
		$directories[$dirname] = $directory;
	}
	ksort($directories);
	
	if($dir_id == 0) {
		$directory = array("dir_id" => 0, "dirname" => $_CONF["locale"]->GetString("home"), "icon" => "home.gif");
	} else {
		$result = $_DATABASE->Query("SELECT * from soybo_directories WHERE dir_id='$dir_id'");
		$directory = mysql_fetch_assoc($result);		
	}
	
	$response["template"] = "organize.tpl";
	$response["type"] = "user";
	$response["user_id"] = $user_id;
	$response["username"] = $username;
	$response["services"] = $userServices;
	$response["groupServices"] = $groupServices;
	$response["services_remaining"] = $remainingServices;
	$response["subdirectories"] = $directories;
	$response["directory"] = $directory;

	//echo_object(0, $response, "response");
	
	return ScriptResponse($response);
}









/* Function:	Generate_Pending_List
 * 
 * Description: Generates the list of pending services (for installation)
 *
 * Parameters:	None
 *
 * Returns:		array of pending services
 *
 * Revision History:
 *
 *	2003-03-18 - Initial version (adam)
 *
 *
 */

function Generate_Pending_List($services)
{
	global $_CONF;
	global $configuration, $current_element, $current_data, $array_key;
	
		// Get a list of uninstalled services
	
	$pending = array();
	$path_services = $_CONF["path_services"];
	
		// This should never happen, but we'll error out if the services directory could not be found
		
	if(!file_exists($path_services)) die("Services directory not found!");
	
		// Get a directory handle
	
	$dir = dir($path_services);

    while(false !== ($file = $dir->read())) {
        $filepath = $path_services . $file;
        $filetype = filetype($filepath);
        $match = false;
        
        if(substr($file, 0, 1) != "." && $filetype == "dir") {
        
        		// Quick check to see if directory path is contained in one of the existing services
        	
        	foreach($services as $service) {
        		if($service["path"] == $file)
        			$match = true;
        		
        		if($match) break;
        	}
        	if(!$match) {        
				$dir2 = dir($filepath);
		
					// Load the service's XML configuration file
					
				while(false !== ($file2 = $dir2->read())) {
					if($file2 == "soybo.xml") {
						$filename = $filepath . "/" . "soybo.xml";
						
						$xml_parser = xml_parser_create();
						xml_parser_set_option($xml_parser, XML_OPTION_CASE_FOLDING, 0);
						xml_set_element_handler($xml_parser, "_PrvServiceConfStartElement", "_PrvServiceConfEndElement");
						xml_set_character_data_handler($xml_parser, "_PrvServiceConfCharacterData");
					
						if(!($fp = fopen($filename, "r"))) die("could not open XML input");
						
								// Parse the configuration XML document
								
						while ($data = fread($fp, 4096)) {
							if (!xml_parse($xml_parser, $data, feof($fp))) {
								die(sprintf("XML error: %s at line %d",
											xml_error_string(xml_get_error_code($xml_parser)),
											xml_get_current_line_number($xml_parser)));
							}
						}
							
							// Free the parser and close file pointer
							
						xml_parser_free($xml_parser);
						fclose($fp);
						
							// Add to the pending array if the file doesn't already exist in the
							// installed services array
						
						if(!$services[$configuration["service_name"]]) {
							$configuration["path"] = $filename;
							$pending[$configuration["service_name"]] = $configuration;
						}
					}
				}
				$dir2->close();
			}
        }   
    }
    
	//echo_object(0, $pending, "Pending Services");
	
	return $pending;
}


/* Function:    _PrvServiceConfStartElement
 *
 * Description: Start element handler for XML Parsing. Adds elements to
 *              the global configuration array.
 *
 * Parameters:  parser - the XML parser
 *              name - the name of the element
 *              attrs - attributes of the element
 *
 * Returns:     Nothing
 *
 * Revision History:
 *
 *  2003-03-15 - Initial version (adam)
 *
 *
 */

function _PrvServiceConfStartElement($parser, $name, $attrs)
{
    global $current_element, $current_data;

    $current_data = false;
    $current_element = strtolower($name);
}


/* Function:    _PrvServiceConfEndElement
 *
 * Description: End element handler for XML Parsing. Adds the character data
 *              for the current xml element.
 *
 * Parameters:  parser - the XML parser
 *              name - the name of the element
 *
 * Returns:     Nothing
 *
 * Revision History:
 *
 *  2003-03-15 - Initial version (adam)
 *
 *
 */
 
function _PrvServiceConfEndElement($parser, $name)
{
    global $configuration, $current_element, $current_data;
    
    $current_data = trim($current_data);
    
    if($current_data) {
		$configuration[$current_element] = $current_data;
    }
                    
    $current_data = false;
}



/* Function:    _PrvServiceConfCharacterData
 *
 * Description: Processes character data received by the XML parser
 *
 * Parameters:  parser - the XML parser
 *              data - the data received
 *
 * Returns:     Nothing
 *
 * Revision History:
 *
 *  2003-03-15 - Initial version (adam)
 *
 *
 */
 
function _PrvServiceConfCharacterData($parser, $data)
{
    global $current_data;

    $current_data .= $data;
    
}


?>