<?php

/* Copyright (C) 2003 Adam Tow

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
   
   
if(!defined("kUSER_ID_GUEST"))              define("kUSER_ID_GUEST", 1);
if(!defined("kGROUP_ID_ADMINISTRATOR"))     define("kGROUP_ID_ADMINISTRATOR", 1);
if(!defined("kGROUP_ID_GUEST"))             define("kGROUP_ID_GUEST", 2);
if(!defined("kGROUP_ID_MEMBER"))            define("kGROUP_ID_MEMBER", 3);


/* Function:    RunWithParams
 *
 * Description: Entry point for soyvice manages users and groups on the Soybo server.
 *
 * Parameters:  methodName - the method to call
 *              params - parameters to send to the function
 *
 * Returns:     Script response object
 *
 * Revision History:
 *
 *  2003-03-18 - Initial version (adam)
 *
 *
 */
 
function RunWithParams($methodName, $params)
{
    global $_DATABASE;
    global $_CONF;  

        // This method can only be run by users who belong in the Administrator group
    
    switch($methodName)
    {
        case "new_user":
            $response["groups"] = Get_Groups();
            $response["type"] = "new";
            $response["template"] = "detailUser.tpl";
            return ScriptResponse($response);
            break;
        
        case "new_group":
            $response["type"] = "new";
            $response["template"] = "detailGroup.tpl";
            return ScriptResponse($response);
            break;
            
        case "edit_user":
            return Generate_User_Edit($params);
            
        case "edit_group":
            return Generate_Group_Edit($params);
        
        case "delete_user":
            return Generate_User_Delete($params);
            break;
        
        case "delete_group":
            return Generate_Group_Delete($params);
            
        case "confirm_edit_user":
            return User_Edit($params);
            
        case "confirm_new_user":
            return User_New($params);
            
        case "confirm_delete_user":
            return User_Delete($params);
            break;
        
        case "confirm_edit_group":
            return Group_Edit($params);
            
        case "confirm_new_group":
            return Group_New($params);
            
        case "confirm_delete_group":
            return Group_Delete($params);

        default:
            return Generate_Users_Groups($params);
            break;  
    }
    
    return ScriptErrorResponse("", $params["locale"]["errMethodname"]);
}



/* Function:    Get_Groups
 * 
 * Description: Helper function to get an array of group id's and groupnames
 *
 * Parameters:  none
 *
 * Returns:     Script response object
 *
 * Revision History:
 *
 *  2003-03-11 - Initial version (adam)
 *
 */
 
function Get_Groups()
{
        // Get the list of groups
    
    $query = "SELECT * FROM soybo_groups ORDER BY groupname";
    $result = mysql_query($query) or die(mysql_error());
    $groups = array();
        
    while($row = mysql_fetch_assoc($result)) {
        $group_id = $row["group_id"];
        $groupname = $row["groupname"];
        array_push($groups, array("id" => $group_id, "name" => $groupname));
    }
    
    return $groups;
}



/* Function:    User_Delete
 * 
 * Description: This function deletes a user from the database.
 *
 * Parameters:  params - the script parameters
 *
 * Returns:     Script response object
 *
 * Revision History:
 *
 *  2003-03-11 - Initial version (adam)
 *
 */
 
function User_Delete($params)
{
    global $_DATABASE;
    global $_CONF;
    
    if(!$params["cancel"]) {
        $user_id = $params["user_id"];
    
        if(!$user_id)
            return ScriptErrorResponse("", $params["locale"]["errInvalidParameters"]);
        
            // Make sure we are not deleting the main admin or guest account
            
        if($user_id == kUSER_ID_GUEST) {
            return ScriptErrorResponse("", $params["locale"]["errDeleteAdminGuest"]);
        }

            // Move the user files to the trash users directory
        
        $username = $_DATABASE->Get_Username($user_id);
        
            // Make sure we're not deleting ourselves!
        
        if($username == $_CONF["username"]) return ScriptErrorResponse("", $params["locale"]["errDeleteSelf"]); 
        
            // Create the trash and trash/users directories if necessary
            
        $trashpath = $_CONF["path"] . "/trash";
        if(!file_exists($trashpath)) {
            $err = mkdir($trashpath, 0777);
            if(!$err) return ScriptErrorResponse("", $params["locale"]["errMkdir"]);
        }
        
        $trashpath .= "/users";
        if(!file_exists($trashpath)) {
            $err = mkdir($trashpath, 0777);
            if(!$err) return ScriptErrorResponse("", $params["locale"]["errMkdir"]);
        }
                        
        $userpath = $_CONF["path_users"] . "$username";
        
        if(file_exists($userpath)) {
            $newpath = $trashpath . "/$username";
            
            // If the file exists, we'll create a temp name for the moved directory
            
            if(file_exists($newpath)) {
                $newpath .= "_" . time();
    
                    // This should never happen, but if it does, exit gracefully and allow
                    // the user to try again (when the value for $newpath will not result in an existing file!)
                            
                if(file_exists($newpath)) 
                    return ScriptErrorResponse("", $params["locale"]["errDeleteFileExists"]);
            }
                // Move the directory
        
            rename($userpath, $newpath);
        }
        
            // Delete the user's access rules, services, groups, and directories
                        
        $_DATABASE->Query("DELETE FROM soybo_access_client WHERE type='1' AND user_group_id='$user_id'");
        $_DATABASE->Query("DELETE FROM soybo_access_ip WHERE type='1' AND user_group_id='$user_id'");
        $_DATABASE->Query("DELETE FROM soybo_services_users_groups WHERE type='1' AND user_group_id='$user_id'");
        $_DATABASE->Query("DELETE FROM soybo_directories WHERE type='1' AND user_group_id='$user_id'");
               
            // Delete the user, sessions, and workspaces
        
        $_DATABASE->Query("DELETE FROM soybo_workspaces WHERE user_id='$user_id'");
        $_DATABASE->Query("DELETE FROM soybo_sessions WHERE user_id='$user_id'");
        $_DATABASE->Query("DELETE FROM soybo_users WHERE user_id='$user_id'");
        
            // Log the event
            
        GFn_Log_Event(kLOG_ADMIN, "User Deleted: $username");
        
        $response["new_path_user"] = $newpath;
        $response["template"] = "success_delete_user.tpl";
        
        return ScriptResponse($response);
        
    } else {
        return Generate_Users_Groups($params);
    }   
}


/* Function:    User_New
 * 
 * Description: This function adds a new user to the database
 *
 * Parameters:  params - the parameters to the script
 *
 * Returns:     Script response object
 *
 * Revision History:
 *
 *  2003-03-11 - Initial version (adam)
 *
 */
 
function User_New($params)
{
    global $_CONF;
    global $_DATABASE;

    $username = $params["username"];
    $group_id = $params["group_id"];
    $firstname = $params["firstname"];
    $lastname = $params["lastname"];
    $email = $params["email"];
    $password = $params["password"];
    $confirmPass = $params["confirm_password"];
    $groupname = $_DATABASE->Get_Groupname($group_id);
    
        // Validate the parameters
        
    if(!$username) return ScriptErrorResponse("", $params["locale"]["errInvalidParameters"]);
    
        // Username 32 characters or less?
        
    if(strlen($username) > 32) return ScriptErrorResponse("", $params["locale"]["errNameInvalid"]);
    if(strlen($username) < 3) return ScriptErrorResponse("", $params["locale"]["errNameInvalid"]);
    
        // Only alphanumeric characters?
        
    if($username != ereg_replace("[^[:alnum:]_-]", "", $username))
        return ScriptErrorResponse("", $params["locale"]["errNameInvalid"]);
    
        // Duplicate username?
        
    $user_id = $_DATABASE->Get_User_ID($username);
    if($user_id) return ScriptErrorResponse("", $params["locale"]["errDuplicateUsername"]);
    
        // Validate the user's information
    
    $err = Validate_User_Info($params);
    if($err) return $err;
    
        // Verify the password
    
    if(!$password || !$confirmPass) return ScriptErrorResponse("", $params["locale"]["errNoPasswords"]);
    if($password != $confirmPass) return ScriptErrorResponse("", $params["locale"]["errPasswordMatch"]);
    if(strlen($password) > 32) return ScriptErrorResponse("", $params["locale"]["errPasswordLength"]);
    if(strlen($password) < 5) return ScriptErrorResponse("", $params["locale"]["errPasswordLength"]);
    
        // Hash the password
        
    $md5password = md5($password);
    
        // Make sure a directory doesn't exist in the users directory with this username
    
    $userpath = $_CONF["path_users"] . $username;

        // Create a directory
            
    if(!file_exists($userpath)) {
	    $err = mkdir($userpath, 0777);
    	if(!$err) return ScriptErrorResponse("", $params["locale"]["errMkdir"]);
    }
        
        // Add the new user into the database
    
    $query2 = "INSERT INTO soybo_users (group_id, username, password, firstname, lastname, email) VALUES ('$group_id', '$username', '$md5password', '$firstname', '$lastname', '$email')";
    $result2 = mysql_query($query2) or die(mysql_error());
    
        // Log the event
    
    GFn_Log_Event(kLOG_ADMIN, "User Created: $username");
    
    $response["groupname"] = $groupname;
    $response["path_user"] = $userpath;
    $response["template"] = "success_new_user.tpl";
    
    return ScriptResponse($response);
}



/* Function:    User_Edit
 * 
 * Description: This function edits a user
 *
 * Parameters:  params - the parameters to the script
 *
 * Returns:     Script response object
 *
 * Revision History:
 *
 *  2003-03-11 - Initial version (adam)
 */
 
function User_Edit($params)
{
    global $_CONF;
    global $_DATABASE;

    $user_id = $params["user_id"];
    $username = $params["username"];
    $group_id = $params["group_id"];
    $firstname = $params["firstname"];
    $lastname = $params["lastname"];
    $email = $params["email"];
    $password = $params["password"];
    $confirmPass = $params["confirm_password"];
    $groupname = $_DATABASE->Get_Groupname($group_id);
    
        // Validate the parameters
    
    if(!$user_id) return ScriptErrorResponse("", $params["locale"]["errInvalidParameters"]);    
    if(!$username) return ScriptErrorResponse("", $params["locale"]["errInvalidParameters"]);
    
        // Username 32 characters or less?
        
    if(strlen($username) > 32) return ScriptErrorResponse("", $params["locale"]["errNameInvalid"]);
    if(strlen($username) < 3) return ScriptErrorResponse("", $params["locale"]["errNameInvalid"]);
    
        // Only alphanumeric characters?
        
    if($username != ereg_replace("[^[:alnum:]_-]", "", $username))
        return ScriptErrorResponse("", $params["locale"]["errNameInvalid"]);
    
        // Username doesn't match user id?
        
    if($user_id != $_DATABASE->Get_User_ID($username))
        return ScriptErrorResponse("", $params["locale"]["errInvalidParameters"]);
    
        // Validate the user's information
    
    $err = Validate_User_Info($params);
    if($err) return $err;
    
        // Verify the password only if the user changed it

    if($password && $confirmPass) {
        if($password != $confirmPass) return ScriptErrorResponse("", $params["locale"]["errPasswordMatch"]);
        if(strlen($password) > 32) return ScriptErrorResponse("", $params["locale"]["errPasswordLength"]);
        if(strlen($password) < 5) return ScriptErrorResponse("", $params["locale"]["errPasswordLength"]);
        
            // Hash the password
            
        $md5password = md5($password);
        
    } else {
        $md5password = false;
    }
        
        // Log the user out by deleting his or her session and workspace

    $_DATABASE->Query("DELETE FROM soybo_sessions WHERE user_id='$user_id'");
    $_DATABASE->Query("DELETE FROM soybo_workspaces WHERE user_id='$user_id'");

        // Update the user in the database
        
    if($md5password) {
        $_DATABASE->Query("UPDATE soybo_users SET group_id='$group_id', password='$md5password', firstname='$firstname', lastname='$lastname', email='$email' WHERE user_id='$user_id'");
    } else {
        $_DATABASE->Query("UPDATE soybo_users SET group_id='$group_id', firstname='$firstname', lastname='$lastname', email='$email' WHERE user_id='$user_id'");
    }
    
        // Log the event
    
    GFn_Log_Event(kLOG_ADMIN, "User Edited: $username");
    
    $response["groupname"] = $groupname;
    $response["path_user"] = $userpath;
    $response["template"] = "success_edit_user.tpl";
    
    return ScriptResponse($response);
    
    
}



/* Function:    Validate_User_Info
 * 
 * Description: Validates user information
 *
 * Parameters:  params - the parameters to the script
 *
 * Returns:     Script response object
 *
 * Revision History:
 *
 *  2003-03-11 - Initial version (adam)
 *
 */
 
function Validate_User_Info($params)
{
    global $_CONF;
    global $_DATABASE;
    
    $username = $params["username"];
    $group_id = $params["group_id"];
    $firstname = $params["firstname"];
    $lastname = $params["lastname"];
    $email = $params["email"];
    $groupname = $_DATABASE->Get_Groupname($group_id);
    
        // Verify the chosen group

    if(!$group_id) return ScriptErrorResponse("", $params["locale"]["errGroupDoesNotExist"]);
    
        // Verify the firstname and lastname fields

    if($firstname) {
        if(strlen($firstname) > 32) return ScriptErrorResponse("", $params["locale"]["errNameInvalid"]);
    }
    
    if($lastname) {
        if(strlen($lastname) > 32) return ScriptErrorResponse("", $params["locale"]["errNameInvalid"]);        
    }
    
        // Verify the email address
    
    if(!$email) return ScriptErrorResponse("", $params["locale"]["errNoEmail"]);
    if(!eregi("^[0-9a-z_]([-_.]?[0-9a-z])*@[0-9a-z][-.0-9a-z]*\\.[a-z]{2,4}[.]?$", $email, $pieces))
         return ScriptErrorResponse("", $params["locale"]["errEmailInvalid"]);
    
        // No error
        
    return false;
}



/* Function:    Group_Delete
 * 
 * Description: This function deletes a group from the database.
 *
 * Parameters:  params - the script parameters
 *
 * Returns:     Script response object
 *
 * Revision History:
 *
 *  2003-03-11 - Initial version (adam)
 *
 */
 
function Group_Delete($params)
{
    global $_CONF;
    global $_DATABASE;

    if(!$params["cancel"]) {
        $group_id = $params["group_id"];
        $transfer_id = $params["transfer_id"];
                
        if($group_id && $transfer_id && $group_id != $transfer_id) {
            
                // Make sure the transferGroup exists
                
            $groupname = $_DATABASE->Get_Groupname($group_id);
            
            if($groupname) {
            
                    // We don't delete the Administrator, Member, or Guest groups
                
                if($group_id != kGROUP_ID_ADMINISTRATOR && $group_id != kGROUP_ID_MEMBER && $group_id != kGROUP_ID_GUEST) { 
                    
                    $transfer_name = $_DATABASE->Get_Groupname($transfer_id);
                
                    if($transfer_name) {
                   
                            // Change the group of all members of the deleted group to the transfer group
                        
                        $_DATABASE->Query("UPDATE soybo_users SET group_id='$transfer_id' WHERE group_id='$group_id'");
                        
                            // Remove references to the group from the access rules, directories, and services tables and finally the group itself
                        
                        $_DATABASE->Query("DELETE FROM soybo_access_ip WHERE type='2' AND user_group_id='$group_id'");
                        $_DATABASE->Query("DELETE FROM soybo_access_client WHERE type='2' AND user_group_id='$group_id'");
                        $_DATABASE->Query("DELETE FROM soybo_directories WHERE type='2' AND user_group_id='$group_id'");
                        $_DATABASE->Query("DELETE FROM soybo_services_users_groups WHERE type='2' AND user_group_id='$group_id'");
                        $_DATABASE->Query("DELETE FROM soybo_groups WHERE group_id='$group_id'");						
                        
                            // Log the event
                        
                        GFn_Log_Event(kLOG_ADMIN, "Deleted Group: $groupname");
                        
                        $response["new_path_group"] = $newpath;
                        $response["groupname"] = $groupname;
                        $response["template"] = "success_delete_group.tpl";
                        
                        return ScriptResponse($response);
                        
                    } else {
                        return ScriptErrorResponse("", $params["locale"]["errTransferGroupDoesNotExist"]);
                    }
                } else {
                    return ScriptErrorResponse("", $params["locale"]["errDeleteDefaultGroups"]);
                }
            } else {
                return ScriptErrorResponse("", $params["locale"]["errGroupDoesNotExist"]);
            }
        } else {
            return ScriptErrorResponse("", $params["locale"]["errInvalidParameters"]);
        }
    } else {
        return Generate_Users_Groups($params);
    }
}


/* Function:    Group_New
 * 
 * Description: This function adds a new group to the database
 *
 * Parameters:  params - the parameters to the script
 *
 * Returns:     Script response object
 *
 * Revision History:
 *
 *  2003-03-11 - Initial version (adam)
 *
 */
 
function Group_New($params)
{
    global $_CONF;
    global $_DATABASE;
    
    $groupname = $params["groupname"];
    
        // Was something entered?

    if(!$groupname) return ScriptErrorResponse("", $params["locale"]["errNoGroupName"]);

        // Group name 32 characters or less?
        
    if(strlen($groupname) > 32) return ScriptErrorResponse("", $params["locale"]["errNameInvalid"]);
    if(strlen($groupname) < 3) return ScriptErrorResponse("", $params["locale"]["errNameInvalid"]);
    
        // Only alphanumeric characters?
        
    if($groupname != ereg_replace("[^[:alnum:]_-]", "", $groupname)) return ScriptErrorResponse("", $params["locale"]["errNameInvalid"]);
    
        // Duplicate group name?
    
    $result = $_DATABASE->Query("SELECT * FROM soybo_groups WHERE groupname='$groupname'");
    
    if(mysql_num_rows($result) == 0) {
        $result = $_DATABASE->Query("INSERT INTO soybo_groups (groupname) VALUES ('$groupname')");
                
            // Log the event
        
        GFn_Log_Event(kLOG_ADMIN, "Group Created: $groupname");
        
    } else {
        return ScriptErrorResponse("", $params["locale"]["errDuplicateGroupName"]);
    }
    
    return Generate_Users_Groups($params);
}



/* Function:    Group_Edit
 * 
 * Description: This function edits the name of an existing group
 *
 * Parameters:  params - the parameters to the script
 *
 * Returns:     Script response object
 *
 * Revision History:
 *
 *  2003-03-11 - Initial version (adam)
 *
 */
 
function Group_Edit($params)
{
    global $_CONF;
    global $_DATABASE;
    
    $originalGroupName = $params["originalname"];
    $groupname = $params["groupname"];
        
        // Don't do anything if the user didn't edit anything
        
    if($groupname == $originalGroupName)
        return Generate_Users_Groups($params);
    
        // Was something entered?

    if(!$groupname) {
        return ScriptErrorResponse("", $params["locale"]["errNoGroupName"]);
    }

        // Group name 32 characters or less?
        
    if(strlen($groupname) > 32 && strlen($groupname) > 3) {
        return ScriptErrorResponse("", $params["locale"]["errNameInvalid"]);
    }
    
        // Only alphanumeric characters?
        
    if($groupname != ereg_replace("[^[:alnum:]_-]", "", $groupname)) {
        return ScriptErrorResponse("", $params["locale"]["errNameInvalid"]);
    }
    
        // Duplicate group name?
        
    $result = $_DATABASE->Query("SELECT * FROM soybo_groups WHERE groupname='$groupname'");
    
    if(mysql_num_rows($result) == 0) {
            
            // Add the new group into the database
            
        $_DATABASE->Query("UPDATE soybo_groups SET groupname='$groupname' WHERE groupname='$originalGroupName'");
        
            // Log the event
        
        GFn_Log_Event(kLOG_ADMIN, "Group Edited: $groupname");
        
    } else {
        return ScriptErrorResponse("", $params["locale"]["errDuplicateGroupName"]);
    }
    
    return Generate_Users_Groups($params);
}



/* Function:    Generate_User_Edit
 * 
 * Description: This function generates the data for the edit users template
 *
 * Parameters:  params - the script parameters
 *
 * Returns:     Script response object
 *
 * Revision History:
 *
 *  2003-03-11 - Initial version (adam)
 *
 */
 
function Generate_User_Edit($params)
{
    global $_DATABASE;
    
    $user_id = $params["user_id"];
    
    if(!$user_id)
        return ScriptErrorResponse("", $params["locale"]["errInvalidParameters"]);
    
    $userinfo = $_DATABASE->Get_Row("soybo_users", "*", "user_id", $user_id);

    $response["groups"] = Get_Groups();
    $response["user"] = $userinfo;
    $response["type"] = "edit";
    $response["template"] = "detailUser.tpl";
    
    return ScriptResponse($response);
}


/* Function: Generate_Group_Edit
 * 
 * Description: This function generates the data for the edit groups template
 *
 * Parameters:  params - the script parameters
 *
 * Returns:     Soybo XML Response
 *
 * Revision History:    3/11/2003 - created adam
 *
 */
 
function Generate_Group_Edit($params)
{
    global $_DATABASE;
    
    $group_id = $params["group_id"];
        
        if(!$group_id)
            return ScriptErrorResponse("", $params["locale"]["errInvalidParameters"]);
            
        $groupname = $_DATABASE->Get_Groupname($group_id);
        
        if(!$groupname)
            return ScriptErrorResponse("", $params["locale"]["errInvalidParameters"]);
        
        $response["originalname"] = $groupname;
        $response["type"] = "edit";
        $response["template"] = "detailGroup.tpl";
        
        return ScriptResponse($response);
}



/* Function:    Generate_User_Delete
 * 
 * Description: This function generates the data for the user delete template
 *
 * Parameters:  params - the script parameters
 *
 * Returns:     Script response object
 *
 * Revision History:
 *
 *  2003-03-11 - Initial version (adam)
 *
 */
 
function Generate_User_Delete($params)
{
    global $_DATABASE;
    
    $user_id = $params["user_id"];

    if(!$user_id)
        return ScriptErrorResponse("", $params["locale"]["errInvalidParameters"]);
    
        // Make sure we are not deleting the main admin or guest account
        
    if($user_id == kUSER_ID_GUEST) {
        return ScriptErrorResponse("", $params["locale"]["errDeleteGuest"]);
    }
    
    $username = $_DATABASE->Get_Username($user_id);
    
    $response["user_id"] = $user_id;
    $response["username"] = $username;
    $response["template"] = "deleteUser.tpl";
    
    return ScriptResponse($response);
}




/* Function:    Generate_Group_Delete
 * 
 * Description: This function generates the data for the delete_groups template
 *
 * Parameters:  params - the script parameters
 *
 * Returns:     Script response object
 *
 * Revision History:
 *
 *  2003-03-11 - Initial version (adam)
 *
 */
 
function Generate_Group_Delete($params)
{
    global $_DATABASE;
    
    $groupToDeleteID = $params["group_id"];
    $groupToDeleteName = false;
    
        // Get the list of groups
    
    $query = "SELECT * FROM soybo_groups ORDER BY groupname";
    $result = mysql_query($query) or die(mysql_error());
    $groups = array();
        
    while($row = mysql_fetch_assoc($result)) {
        $group_id = $row["group_id"];
        $groupname = $row["groupname"];
        
        if($groupToDeleteID != $group_id) {
            array_push($groups, array("id" => $group_id, "name" => $groupname));
        } else {
            $groupToDeleteName = $groupname;
        }
    }
    
    if(!$groupToDeleteName)
        return ScriptErrorResponse("", $params["locale"]["errInvalidParameters"]);
    
    $response["groups"] = $groups;
    $response["group_id"] = $groupToDeleteID;
    $response["groupname"] = $groupToDeleteName;
    $response["template"] = "deleteGroup.tpl";
    
    return ScriptResponse($response);
}



/* Function: Generate_Users_Groups
 * 
 * Description: This function generates the user and group list that will be displayed in the
 *              default template.
 *
 * Parameters:  params - the parameters to the script
 *
 * Returns:     Soybo XML Response
 *
 * Revision History:    3/11/2003 - created adam
 *
 */
 
function Generate_Users_Groups($params)
{
    global $_DATABASE;
    
        // Get the list of groups
    
    $query = "SELECT * FROM soybo_groups ORDER BY groupname";
    $result = mysql_query($query) or die(mysql_error());
    $groups = array();
    $groupnames = array();
        
    while($row = mysql_fetch_assoc($result)) {
        $group_id = $row["group_id"];
        $groupname = $row["groupname"];
        
        $groupnames[$group_id] = $groupname;
        
            // Calculate the number of users in the group
    
        $query2 = "SELECT * FROM soybo_users WHERE group_id='$group_id'";
        $result2 = mysql_query($query2) or die(mysql_error());
        $numUsers = mysql_num_rows($result2);
        
        array_push($groups, array("id" => $group_id, "name" => $groupname, "num" => $numUsers));
    }
    
        // Get the list of users
    
    $query = "SELECT * FROM soybo_users WHERE user_id<>'1' ORDER BY username";
    $result = mysql_query($query) or die(mysql_error());
    $users = array();
    
    while($row = mysql_fetch_assoc($result)) {      
        array_push($users, array(   "id" =>  $row["user_id"],
                                    "name" => $row["username"],
                                    "group" => $groupnames[$row["group_id"]],
                                    "email" => $row["email"]));
    }

    $response["groups"] = $groups;
    $response["users"] = $users;
    $response["template"] = "users_and_groups.tpl";
    
    return ScriptResponse($response);
}


?>