<?php

/* Copyright (C) 2003 Adam Tow

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
   

global $_CONF;

if(file_exists("./conf_global.php"))
    require_once "./conf_global.php";
    
if(!$_CONF) die("Please run the Soybo installer before proceeding.");

$_CLIENTS = array();

    // Start the timer
    


require_once "./source/functions.php";
require_once "./source/session.php";                // Session Module
require_once "./source/constants.php";
require_once "./source/logs.php";                   // Logging Module
require_once "./source/preferences.php";

require_once "./support/thirdparty/xmlrpc.php";     // XML-RPC Third Party Code

require_once "./source/localization.class.php";     // Localization
require_once "./source/page.class.php";             // Page Module
require_once "./source/workspace.class.php";        // Workspace Module
require_once "./source/client.class.php";           // Client Class
require_once "./source/server.class.php";           // Server Module

    // Register sessions
    
session_register("session_id");

    // Global Variables


$_LOCALIZATION = array();
$_DATABASE = new Database;
$_PAGE = new Page;
$_SOYBO = new Soybo;


class Soybo
{
    var $server;
    var $client;


    /* Soybo Constructor
     *
     * Create references to a server and client object.
     *
     */
     
    function Soybo()
    {                           
        $this->Initialize();
        $this->Get_Locale();
        $this->Get_Server();
        $this->Get_Client();
    }
    
    function Initialize()
    {
        GFn_Load_Config_Locales();
        GFn_Load_Authenticators();
        GFn_LoadScripts("/support/scripts_handlers/");
        GFn_Load_Custom_Clients();
    }
    
    
    /* Get_Client
     * 
     * Identifies the device that the user is using to
     * access the system and instantiates the appropriate
     * client class object.
     *
     * Searches in the following order: 
     *  - custom clients
     *  - phone clients
     *  - handheld clients
     *  - XML-RPC and SOAP Clients
     *
     * Desktop client is automatically instantiated if no match
     * is found.
     *
     *
     */

    function Get_Client()
    {
        global $_CONF;
        global $_DATABASE;
        
        $useragent = $_SERVER["HTTP_USER_AGENT"];
        
        if($_SERVER["REQUEST_METHOD"] == "POST" && $_SERVER["CONTENT_TYPE"] == "text/xml") {
        
                // Create an XML-RPC Client
            
            $className = "XMLRPC_Client";
        
            $this->client = new $className;
            $this->client->server = $this->server;
            $this->client->Main();
        } else {
            if($_CONF["allow_http_access"]) {
                $session_id = $_SESSION["session_id"];
                
                if($session_id) {
                    $session_state = $this->server->Session_Valid($session_id);
                
                        // If the session is valid, get the client class name from
                        // the user's preferences for this particular client. Otherwise,
                        // go with the default client class for the given client
                        
                    if($session_state == kSESSION_VALID) {
                        $_CONF["client_classname"] = $this->server->Get_User_Client_Classname($session_id, $useragent);
                    }
                } else {
                    $session_state = kSESSION_NONE;
                }
                
                    // If the client class name has not been defined, get the default
                
                if(!$_CONF["client_classname"])
                    $_CONF["client_classname"] = $this->server->Get_Client_Classname($useragent);
                
                $this->client = new $_CONF["client_classname"];
                $this->client->server = $this->server;
                $this->client->Main($session_id, $session_state);
            } else {
                die("HTTP/GET/POST Access is not allowed on this Soybo server.");   
            }
        }
        
    }
    
    
    
    /* Get_Server
     * 
     * Identifies the server that the user is using to
     * access the system and instantiates the appropriate
     * server class object.
     *
     *
     */
    
    function Get_Server()
    {
        global $_SERVERS;
        
        $serverName = false;
        $serverSoftware = $_SERVER["SERVER_SOFTWARE"];
                
            // Identify what server we're running on
            
        GFn_Load_Custom_Servers();

        foreach($_SERVERS as $name => $className) {
            $pos = strpos($serverSoftware, $name);
            
            if($pos !== false) {
                $serverName = $className;
                break;
            }
        }
        
        if(!$serverName)
            $serverName = "Server";
            
        $this->server = new $serverName;
    }
    
    
    /* Get_Locale
     * 
     * Retrieves the requested locale or defaults to the default locale.
     *
     */ 
    
    function Get_Locale()
    {
        global $_CONF;
        global $_LOCALIZATION;      
        //global $locale;
        
        $localeName = strtolower($_REQUEST["locale"]);
    
        if(!$localeName || $localeName == "default") {
            $localeName = $_CONF["default_locale_name"];
        }
        
        $_CONF["locale_name"] = $localeName;
        
        $localeInfo = $_LOCALIZATION[$localeName];
        
        //echo_object(0, $localeInfo, "Locale");
        
        if(!$localeInfo) { // ($localeName != $_CONF["default_locale_name"])) {
        
            $localeName = $_CONF["default_locale_name"];
            $localeInfo = $_LOCALIZATION[$localeName];
            
            if(!$localeInfo) 
                die("Localization load failed.");
        }
        $_CONF["locale"] = new Locale($localeName, $localeInfo["path"]);
    }
}



?>