<?php

/* Copyright (C) 2003 Adam Tow

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
   
   
global $_CONF;

require_once $_CONF["path_html"] . "/support/clients/xmlrpc.class.php";      // XML-RPC Client
require_once $_CONF["path_html"] . "/support/clients/http.class.php";        // HTTP Client Class
require_once $_CONF["path_html"] . "/support/clients/phone.class.php";       // Phone Client
require_once $_CONF["path_html"] . "/support/clients/handheld.class.php";    // Handheld Client
require_once $_CONF["path_html"] . "/support/clients/desktop.class.php";     // Desktop Client
require_once $_CONF["path_html"] . "/support/clients/wap.class.php";         // WAP Client


/* Class:       Client
 *
 * Description: Default client class.
 *
 * Revision History: 2003-03-15 - adam
 *
 *
 */

class Client
{       
        // Client information
        
    var $name;
    var $type;
    
        // Session and workspace Information

    var $locale;        
    var $home;
    var $currPath;
    var $currScript;
    var $server;
    var $workspace;


    /* Method:      Client
     *
     * Description: Class constructor method
     *
     * Parameters:  None
     *
     * Returns:     true
     *
     * Revision History: 2003-03-15 - adam
     *
     */
     
    function Client()
    {   
        $this->name = "Client";
        $this->type = "Client";
        
        $this->workspace = new Workspace;
        
        return true;    
    }
    

    /* Method:      Generate_Service_Media
     *
     * Description: Creates a user's media path in the soybo_www/media directory. The path to the
     *              user's directory is md5'ed with the username and soybo_data path to create a unique
     *              identifier for the user's media directory
     *
     *              i.e.    /media/xxx/services
     *                      /media/xxx/directories
     *
     * Parameters:  name - the name of the directory to create
     *              path - the path to the directory containing the media files
     *
     * Returns:     the path
     *
     * Revision History: 2003-03-15 - adam
     *
     */
     
    function Generate_Service_Media($name, $path)
    {
        global $_CONF;

        $hashString = md5($_CONF["username"] . $_CONF["path"]);
        
            // Now check to see if the media directory is present in the www/media directory
            // If not, copy the files in the service's media directory there
        
        $mediaPath = $_CONF["path_html"] . "/media";
        if(!file_exists($mediaPath)) {
            mkdir($mediaPath, 0777);
        }
                
        $userMediaPath = $mediaPath . "/" . $hashString;
        if(!file_exists($userMediaPath)) {
            mkdir($userMediaPath, 0777);
        }
        
        $wwwMediaPath = $userMediaPath . "/$name";
        $theMediaPath = $path . "soybo_media";
                    
        if(!file_exists($wwwMediaPath) && file_exists($theMediaPath)) {
            GFn_Directory_Copy($theMediaPath, $wwwMediaPath);
        }
        
        return true;
    }   


    
    
    /* Method:      Media_Path_Generate
     *
     * Description: Creates a path to the services or directories media directories for the current user
     *
     *              i.e.    /media/john/services
     *                      /media/jean/directories
     *
     * Parameters:  name - the name of the path to generate
     *              absolute - specify true to return an absolute path, false for a relative path (to soybo_www root)
     *
     * Returns:     the path
     *
     * Revision History:
     *
     *  2003-03-22 - Initial version (adam)
     *
     */
        
    function Media_Path_Generate($name, $absolute)
    {
        global $_CONF;
        
        $hashString = md5($_CONF["username"] . $_CONF["path"]);
        
        $basePath = ($absolute) ? $_CONF["path_media"] : $_CONF["uri_media"];
        
        $result = $basePath . $hashString . "/$name";

        return $result;
    }
    

    /* Method:      Media_Images_Generate
     *
     * Description: Generates an array of images for use in tempates
     *
     * Parameters:  None
     *
     * Returns:     Array of image filenames (minus file extensions)
     *
     * Revision History:
     *
     *  2003-03-22 - Initial version (adam)
     *
     */
     
    function Media_Images_Generate()
    {
        global $_CONF;
        
        $images = array();
        $len = count($this->mediaPaths) - 1;
        
        for($i = $len; $i >= 0; $i--) {
            $mediaPath = $this->mediaPaths[$i];
            $imagesPath = $mediaPath . "/images/";
            
            //echo "$imagesPath<br />";
            
            $absoluteImagesPath = $_CONF["path_html"] . $imagesPath;
            
            //echo $absoluteImagesPath . "<br />";
            
            if(file_exists($absoluteImagesPath)) {
                $d = dir($absoluteImagesPath);
        
                while(false !== ($file = $d->read())) {
                    if($file == "." || $file == ".." || substr($file, 0, 1) == ".") {
                        continue;
                    } else {
                        $pathParts = pathinfo($absoluteImagesPath . $file);
                        $basename = basename($pathParts["basename"], "." . $pathParts["extension"]);
                        
                        //echo $basename . "<br />";
                        if(!$images[$basename])
                            $images[$basename] = $_CONF["uri"] . $imagesPath . $file;
                    }
                }
                $d->close(); 
            }
        }
        
        return $images;
    }

}


/* Custom Clients
 *
 * Custom Clients are placed in the /support/clients/custom/ directory
 * This script includes all those files with the extension .php
 * and assumes that the files are correctly formatted (i.e. don't try this
 * on a production system without first testing!).
 *
 * Each Custom Client file will add an entry in the $custom_clients array,
 * which will contain the user-agent string and the name of the class to
 * instantiate if there is a match.
 *
 *
 */


/* Function:    GFn_Load_Custom_Clients()
 * 
 * Description: Scans through all the folders in the /support/clients/customs/ directory
 *              and loads in any PHP file. 
 *
 * Parameters:  None
 *
 * Return:      true
 *
 * Revision History: 2003-03-15 - adam
 *
 */

function GFn_Load_Custom_Clients()
{   
    global $_CONF;
    
    $pathToCustomDevices = $_CONF["path_html"] . "/support/clients/custom/";
    $dir = dir($pathToCustomDevices);
    
    while(false !== ($file = $dir->read())) {
        $filePath = $pathToCustomDevices . $file;
        $filetype = filetype($filePath);
        $pathInfo = pathinfo($filePath);

        if($filetype == "file" && $pathInfo["extension"] == "php") {        
            include($filePath);
        }
    }
    
    return true;
}

?>
