<?php

/* Copyright (C) 2003 Adam Tow

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
   

$timer = new Timer;


/* Class:       Timer
 * 
 * Description: Class to calculate the duration of the script's execution time.
 *
 * Revision History:
 *
 *  2003-03-17 - Initial version (adam)
 *
 * 
 */

class Timer
{
    var $startTime;
    
    function Timer()
    {
        list($usec, $sec) = explode(" ", microtime());
        $this->startTime = (((float) $usec) + (float) $sec);
    }

    function endTimer($echo)
    {
        list($usec, $sec) = explode(" ", microtime());
        $endTime = (((float) $usec) + (float) $sec);
        
        $totalTime = round($endTime - $this->startTime, 4);
        
        if($echo)
            echo "<p align=\"center\"><small>Execution Time: $totalTime seconds</small></p>";
        
        return $totalTime;
    }
}


    ##############################
    #                            #
    #       Error Routines       #
    #                            #
    ##############################


/* Function:    Create_Error
 * 
 * Description: Generates an error array that's used by various functions of the Soybo system
 *
 * Parameters:  faultCode - the error code. Can be omitted.
 *              faultString - the error message.
 *
 * Returns:     an error array
 *
 * Revision History:
 *
 *  2003-03-17 - Initial version (adam)
 *
 * 
 */

function Create_Error($faultCode, $faultString)
{
    return array(false, array(  "faultCode" => $faultCode,
                                "faultString" => $faultString));

}



/* Function: mysql_err
 * 
 * Description: Helper function to give more descriptive information when MySQL chokes.
 *
 * Parameters:  msg - the error message to display
 *              sql - the query string that was used to cause the error
 *              error - MySQL's error message
 *
 */
 
function mysql_err($msg, $sql, $error)
{
    echo "<p>$msg</p>";
    echo "<p>query:<br />$sql</p>";
    echo "<p>error:<br />$error</p>";
    die();
}



    ##############################
    #                            #
    # Directory Copying Routines #
    #                            #
    ##############################



/* Function:    GFn_Copy
 * 
 * Description: Helper function to copy a file from one location to another on the hard drive.
 *
 * Parameters:  oldname - the file location
 *              newname - the new file location
 *
 * Returns:     true
 *
 * Revision History:
 *
 *  2002-11-11 - Initial version (michael at zend dot com)
 *  2003-03-17 - Set file permissions to 0777
 *
 * 
 */
 
function GFn_Copy($oldname, $newname) 
{
    if(is_file($oldname)){

    return copy($oldname, $newname) && chmod($newname, 0777);
    }
    else if(is_dir($oldname)){
        GFn_Directory_Copy($oldname, $newname);
    }
    else{
        die("Cannot copy file: $oldname (it's neither a file nor a directory)");
    }
}



/* Function:    GFn_Directory_Copy
 * 
 * Description: Helper function to copy a directory from one location to another on the hard drive.
 *
 * Parameters:  oldname - the file location
 *              newname - the new file location
 *
 * Returns:     true
 *
 * Revision History:
 *
 *  2002-11-11 - Initial version (michael at zend dot com)
 *  2003-03-17 - Incorporated into Soybo global functions document
 *
 * 
 */
 
function GFn_Directory_Copy($oldname, $newname)
{
    if(!is_dir($newname)){
        mkdir($newname, 0777);
        //chmod($newname, 0777);
    }
    $dir = opendir($oldname);
    while($file = readdir($dir)){
    
            // Also skip hidden files
            
        if($file == "." || $file == ".." || substr($file, 0, 1) == "."){
            continue;
        }
        GFn_Copy("$oldname/$file", "$newname/$file");
    }
    closedir($dir);
    
    return true;
}



    ##############################
    #                            #
    #     Debugging Routines     #
    #                            #
    ##############################

/* Function:    echo_debug
 * 
 * Description: Prints some debugging information if the global variable kDEBUG_ON is set
 *
 * Parameters:  str - the string to display
 *
 * Returns:     true
 *
 * Revision History:
 *
 *  2003-03-17 - Initial version (adam)
 *
 * 
 */
 
function echo_debug($str)
{
    if(kDEBUG_ON)
        echo $str;
        
    return true;
}



/* Function:    echo_object
 * 
 * Description: Useful function for recursively parsing an object and displaying all of its values.
 *
 * Parameters:  level - the level of the function (for indenting purposes)
 *              obj - the object to parse
 *              objName - the name of the object being parsed
 *
 * Returns:     true
 *
 * Revision History:
 *
 *  2003-03-17 - Initial version (adam)
 *
 * 
 */
 
function echo_object($level, $obj, $objName)
{
    if($level == 0)
        echo "<ul type=\"none\">\n";
        
    if(is_array($obj) || is_object($obj)) {
        echo "<li>$objName: {\n";   
        
        if(sizeof($obj) > 0) {
            echo "<ul type=\"none\">";
            while(list($key, $val) = each($obj)) {
                if(is_array($val) || is_object($val)) {
                    echo_object($level + 1, $val, $key);
                } else {
                    if(!$val && $val !== false)
                        echo "<li>$key: 0,</li>\n";
                    else if($val === false)
                        echo "<li>$key: nil,</li>\n";
                    else echo "<li>$key: \"$val\",</li>\n";
                }
            } 
            echo "</ul>\n";
        }
        echo "},\n";
        echo "</li>\n";
    } else {
        echo "<li>$objName: \"$obj\"</li>\n";
    }
    
    if($level == 0)
        echo "</ul>\n";
    
    return true;
}


/* Function:    echo_object
 * 
 * Description: Useful function for recursively parsing an object and displaying all of its values.
 *
 * Parameters:  level - the level of the function (for indenting purposes)
 *              obj - the object to parse
 *              objName - the name of the object being parsed
 *
 * Returns:     true
 *
 * Revision History:
 *
 *  2003-03-17 - Initial version (adam)
 *
 * 
 */

function echo_pre($start)
{
    if(kDEBUG_ON) {
        if($start)
            echo "\n<pre>\n";
        else echo "\n</pre>\n";
    }
}


/* Function:    Cmp_Dirname
 *
 * Description: Function for comparing directory names when sorting
 *
 * Parameters:  dir1 - the first directory
 *              dir2 - the second directory
 *
 * Returns:     0 if equal, 1 if dir1 greater than dir2, else -1
 *
 * Revision History:
 *
 *  2003-03-17 - Initial version (adam)
 *
 * 
 */
 
function Cmp_Dirname($dir1, $dir2) {
    return strcmp($dir1["dirname"], $dir2["dirname"]);
} 


/* Function:    Cmp_Servicename
 *
 * Description: Function for comparing service names when sorting
 *
 * Parameters:  service1 - the first service
 *              service2 - the second service
 *
 * Returns:     0 if equal, 1 if service1 greater than service2, else -1
 *
 * Revision History:
 *
 *  2003-03-17 - Initial version (adam)
 *
 * 
 */
 
function Cmp_Servicename($service1, $service2) {
    return strcmp($service1["name"], $service2["name"]);
} 

?>