<?php

/* Copyright (C) 2003 Adam Tow

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
   

/* Class:       Locale
 * 
 * Description: Class definition for the locale class
 *
 * Revision History:
 *
 *  2003-03-17 - Initial version (adam)
 *
 */ 
 
class Locale
{
    var $strings;
    var $name;
    var $localePath;
    
    
    /* Constructor: Locale
     * 
     * Description: Class Constructor for Locale Class
     *
     * Parameters:  localeName - the name of the locale being generated
     *              localePath - the absolute path to the localization XML file
     *
     * Returns:     nothing
     *
     * Revision History:
     *
     *  2003-03-17 - Initial version (adam)
     *
     */ 
     
    function Locale($localeName, $localePath)
    {   
        $this->strings = array();
        $this->name = $localeName;
        $this->localePath = $localePath;
        $this->LoadLocale($this->localePath);   // Now we load all the strings in the XML Localization file
    }



    /* Method:      LoadLocale
     * 
     * Description: Loads a localization file into the locale object
     *
     * Parameters:  filename - the absolute path to the localization xml file
     *
     * Returns:     true or error if the localization file could not be found.
     *
     * Revision History:
     *
     *  2003-03-17 - Initial version (adam)
     *
     */ 
     
    function LoadLocale($filename)
    {
        global $localeObject;
        
        $localeObject = array();
        
        if($filename) {
            if(!file_exists($filename)) {
                die("Localization file could not be found: $filepath");
            }       
        
                // Include the localization file
                
            include($filename);
            
                // Load the strings from the $locale variable
            
            $this->strings = $locale;
            
            $locale = false;
                        
            return true;
        }
        
        return false;
    }



    /* Method:      GetString
     * 
     * Description: Gets the value of a localized string in the locale object
     *
     * Parameters:  name - the name of the string to retrieve
     *
     * Returns:     the string
     *
     * Revision History:
     *
     *  2003-03-17 - Initial version (adam)
     *
     */ 
            
    function GetString($name)
    {
        return $this->strings[$name];
    }
    
    
    
    /* Method:      SetString
     * 
     * Description: Sets the value of a localized string in the locale object
     *
     * Parameters:  name - the name of the string to modify
     *              value - the new value for the string
     *
     * Returns:     the new string
     *
     * Revision History:
     *
     *  2003-03-17 - Initial version (adam)
     *
     */ 
     
    function SetString($name, $value)
    {
        return $this->strings[$name] = $value;
    }


    
    /* Method:      GetStrings
     * 
     * Description: Returns the strings contained in the locale object
     *
     * Parameters:  None
     *
     * Returns:     An associative array of localization strings
     *
     * Revision History:
     *
     *  2003-03-17 - Initial version (adam)
     *
     */ 
        
    function GetStrings()
    {
        return $this->strings;
    }
}




/* Function:    GFn_Get_Locale
 * 
 * Description: Retrieves the requested locale or defaults to English.
 *
 * Parameters: localeName - the name of the locale to retrieve
 *
 * Returns:     the name of the locale class
 *
 * Revision History:
 *
 *  2003-03-17 - Initial version (adam)
 *
 */ 
    
function GFn_Get_Locale($localeName)
{
    global $_LOCALIZATION;      
    global $_CONF;
    
    if(!$localeName) {
        $localeName = $_CONF["default_locale_name"];
    }
    
    $localeName = strtolower($localeName);
    //echo $localeName;
    
    if($_CONF["locale_name"] != $localeName || !$_CONF["locale"]) { 
        $_CONF["locale_name"] = $localeName;
        $localeInfo = $_LOCALIZATION[$localeName];
        
        if(!$localeInfo) { // || ($localeName != $_CONF["default_locale_name"])) {
            $localeName = $_CONF["default_locale_name"];
            $localeInfo = $_LOCALIZATION[$localeName];

            if(!$localeInfo) 
                die("Localization load failed.");
        }
        
        return new Locale($localeName, $localeInfo["path"]);
    } else {
        //echo "Locale frame already loaded: $localeName<br />";
        
        return $_CONF["locale"];
    }
}



/* Function:    GFn_Load_Config_Locales
 * 
 * Description: Reads all localization configuration files and stores information in the
 *              global $localization array.
 *
 * Parameters:  None
 *
 * Returns:     true
 *
 * Revision History:
 *
 *  2003-03-17 - Initial version (adam)
 *
 *
 */ 
 
function GFn_Load_Config_Locales()
{
    global $_CONF;
    global $_LOCALIZATION;
    global $thisLocale;  
    global $locale; 
    
    $pathToLocales = $_CONF["path_html"] . "/support/localization/";
    
    $d = dir($pathToLocales);
    while(false !== ($file = $d->read())) {
        $locale = false;
        $filepath = $pathToLocales . $file;
        $filetype = filetype($filepath);

        if($file != "." && $file != ".." && $filetype == "file") {
            $pathinfo = pathinfo($filepath);
            $extension = $pathinfo["extension"];
            $localeName = strtolower(basename($pathinfo["basename"], ".$extension"));
            
            if($extension == "php") {
                include($filepath);
                
                $displayName = ($locale["name"]) ? $locale["name"] : $localeName;
                
                $_LOCALIZATION[$localeName] = array("display_name" => $displayName, "path" => $filepath);
            }
        }
    }
    $d->close();

    return true;
}




?>