<?php

/* Copyright (C) 2003 Adam Tow

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
   
   
global $_CONF;

require_once $_CONF["path_smarty"];     // Smarty


/* Class:       Page
 *
 * Description: It extends the Smarty class with methods and slots
 *              pertinent to this application.
 *
 * Revision History:
 *
 *  2003-03-17 - Initial version (adam)
 *
 *
 */

class Page extends Smarty
{
    var $page_head;
    var $page_title;

    /* Constructor: Page
     * 
     * Description: Page class constructor. We set Smarty to force compile (since we are
     *              dynamically generating new templates for each user.
     *
     * Revision History:
     *
     *  2003-03-17 - Initial version (adam)
     *
     *
     */

    function Page()
    {
        $this->Smarty();
        $this->caching = false;
        $this->force_compile = true;
        $this->cache_lifetime = 0;
        $this->debugging = false;
                
        $this->page_title = "";
        $this->page_head = array();
    }



    /* Method:      PageSet
     * 
     * Description: Sets an element to be placed in the page. Right now, we're just using
     *              this to set the title and to set some localization values.
     *
     * Parameters:  type - the element to set
     *              name - the name of the element to set
     *              value - the value to assign to the element
     *
     * Returns:     true
     *
     * Revision History:
     *
     *  2003-03-17 - Initial version (adam)
     *
     *
     */

    function PageSet($type) // ($type, $name, $value) or ($type, $value)
    {
        global $_CONF;
        
        $type = strtolower($type);
        $numArgs = func_num_args();
        $args = func_get_args();

        switch($type)
        {
            case "title":
                $this->page_title = $args[1];
                return $args[1];
                break;

            case "localization":
                $locale = $_CONF["locale"];
                $locale->SetString($args[1], $args[2]);
                
                break;
            
            default: break;
        }
        
        return true;
    }



    /* Method:      PageAdd
     * 
     * Description: Adds an element to the page's head. This is used for stylesheets
     *              so we can properly cascade when displaying the page.
     *
     * Parameters:  type - the element to set
     *              value - the value to assign to the element
     *
     * Returns:     the result of _PageAddHead
     *
     * Revision History:
     *
     *  2003-03-17 - Initial version (adam)
     *
     *
     */

    function PageAdd($type)
    {
        $numArgs = func_num_args();
        $args = func_get_args();
        
        switch($type)
        {
            case "head":
                return $this->_PageAddHead($args[1]);
                break;
                
            default:
                return false;
                break;  
        }
    }



    /* Method:      PageGet
     * 
     * Description: Retrieves the value for a page element
     *
     * Parameters:  type - the element to get
     *              name - the name of the element to get
     *
     * Returns:     the value of the element
     *
     * Revision History:
     *
     *  2003-03-17 - Initial version (adam)
     *
     *
     */

    function PageGet($type) // ($type, $name) or ($type)
    {
        global $_CONF;
        
        $numArgs = func_num_args();
        $args = func_get_args();
        
        switch($type)
        {
            case "title":
                return $this->page_title;
                break;
                        
            case "head":
                return $this->page_head;
                break;
            
            case "localization":
                $locale = $_CONF["locale"];
                return $locale->GetString($args[1]);
                break;
            
            default: break;
        }
    }



    /* Method:      Add_Head_Element
     *
     * Description: Private method to add an element to the head of the page.
     *
     * Parameters:  value   -   The value to be added.
     *
     * Returns:     an array containing the name, value pair
     *
     * Revision History:
     *
     *  2003-03-17 - Initial version (adam)
     *
     *
     */

    function _PageAddHead($value)
    {
        array_push($this->page_head, $value);
        
        return current($this->page_head);
    }
}