<?php

/* Copyright (C) 2003 Adam Tow

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
   

/* Class:       Server
 *
 * Description: The Soybo Server class handles calls for user authentication,
 *              session management, and Soyvice execution.
 *
 * Revision History:
 *
 *  2003-03-15 - Initial version (adam)
 *
 *
 */

global $_CONF;

require_once $_CONF["path_html"] . "/source/scripts.class.php";
require_once $_CONF["path_html"] . "/source/database.class.php";            // Database Module
require_once $_CONF["path_html"] . "/source/authenticator.class.php";       // Authentication Modules

    // For custom servers (i.e. Mac, Windows, *NIX)

$_SERVERS = array();

class Server
{
    var $name;
    
        // Configuration XML File Reference
    
    var $configFile;
    
    function Server()
    {       
        $this->name = "Generic Server";
    }


    
    /* Method:      Login
     * 
     * Description: Authenticates the user and accesses the user account.
     *
     * Parameters:  authenticationRequest - array of authentication information
     *
     * Returns:     success or error array
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */
     
    function Login($authenticationRequest)
    {
        global $_CONF;
        
        if($authenticationRequest["username"] == "guest" && $_CONF["allow_guest_access"]) {
            $result = $this->Verify_User_IP_Address($authenticationRequest);
            
            if($result[0]) {    
                $result = array(true, true);
            }
        } else {
            $result = $this->Authenticate($authenticationRequest);          
            
            if($result[0]) {
                $result = $this->Verify_User_IP_Address($authenticationRequest);
            }
        }
        
        if($result[0]) {
            $session_id = $_SESSION["session_id"];
            
                // Create a session id if an XMLRPC Request is requested.
                
            if(!$session_id) {
                $session_id = session_id();
                $_SESSION["session_id"] = $session_id;
            }
            
                // The login was successful.
                // 1. Create a new row in the sessions table to store the user_id, ip_address, and session_id
                // 2. Retrieve the user's home object
            
            $ip_address = $_SERVER["REMOTE_ADDR"];
            $username = $authenticationRequest["username"];
            
                // Now we store this session information into the database
            
            $this->Session_Generate($session_id, $ip_address, $username);
            
            return array(true, $this->Retrieve_Home($username));
            
        } else {
            $faultCode = $result[1]["faultCode"];
            $faultString = $result[1]["faultString"];
                        
            return Create_Error($faultCode, $faultString);
        }
    }


    
    /* Method:      Authenticate
     * 
     * Description: Launches the appropriate authentication handler
     *
     * Parameters:  authenticationRequest - array containing authentication information
     *
     * Returns:     the result of the authentication
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */
     
    function Authenticate($authenticationRequest)
    {
        global $_AUTHENTICATORS;    // defined in authenticators.php
        global $_CONF;
        
        $loginType = $authenticationRequest["auth"];

        if(!$loginType)
            $loginType = "Basic";
            
        $authenticatorName = str_replace(" ", "_", $loginType);

                // A list of authenticators and their class names has
                // already been loaded in the $_AUTHENTICATORS array.

        if(array_key_exists($authenticatorName, $_AUTHENTICATORS)) {
        
                // Now create an instance of the authenticator
                    
            $authenticator = new $_AUTHENTICATORS[$authenticatorName];
                    
            return $authenticator->Authenticate($authenticationRequest);
        } else {

            
            $err = $_LOCALE->GetString("errLoginTypeUnsupported");
            
            GFn_Log_Event(kLOG_ERROR, "Unsupported authentication type: $loginType");
            
            return Create_Error("", $err);
        }
    }
    


    /* Method:      Verify_User_IP_Address
     * 
     * Description: Checks the Soybo Access Rules to see if the user is
     *              allowed to access the system from this IP Address.
     *
     * Parameters:  authenticationRequest - array containing authentication information
     *
     * Returns:     array of true values or error response array
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */
    
    function Verify_User_IP_Address($authenticationRequest)
    {
        global $_DATABASE;
        global $_CONF;
        
        $remoteAddr = $_SERVER["REMOTE_ADDR"];
        $username = $authenticationRequest["username"];
        $user_id = $_DATABASE->Get_User_ID($username);
        $group_id = $_DATABASE->Get_Group_ID_From_User_ID($user_id);
        $groupname = $_DATABASE->Get_Groupname($group_id);
        
        $err = false;
        
        //echo "Authenticating Remote Address: $remoteAddr<br />";
        //echo "UserID: $user_id GroupID: $group_id<br />";
        
            // Explode the remote IP Address into its parts
            
        $ip_parts = explode(".", $remoteAddr);
        
            // Perform a search on the soybo_access_ip table for IP Addresses matching the remote address
            // We start with the actual IP Address of the request and become more general
            // we each iteration. If the user is allowed, we immediately return true, even if 
            // there is an access entry where the user might be banned. This will allow the creation
            // of a sanctuary computer within a banned domain.
            
        for($i = 4; $i >= 0; $i--) {
            if($i == 0) $addr = "*";
            else if($i == 1) $addr = $ip_parts[0] . ".*";
            else if($i == 2) $addr = $ip_parts[0] . "." . $ip_parts[1] . ".*";
            else if($i == 3) $addr = $ip_parts[0] . "." . $ip_parts[1] . "." . $ip_parts[2] . ".*";
            else if($i == 4) $addr = $remoteAddr;
                        
                // First we see if the user is on the allow list
            
            $query = "SELECT * FROM soybo_access_ip WHERE ip_address='$addr' AND type='1' AND user_group_id='$user_id' AND access='2'";
            $result = $_DATABASE->Query($query);
            
            if($result && ($num = mysql_num_rows($result) > 0)) {
                GFn_Log_Event(kLOG_USER, "Login");
                return array(true, true);
            }

                // Now we check to see if the user is on the banned list
            
            $query = "SELECT * FROM soybo_access_ip WHERE ip_address='$addr' AND type='1' AND user_group_id='$user_id' AND access='1'";
            $result = $_DATABASE->Query($query);
            
            if($result && ($num = mysql_num_rows($result) > 0)) {   
                $err = $_CONF["locale"]->GetString("errPermission");
                $logErr = "Access not allowed for \"$username\" on IP Address: $addr";
                break;
            }

                // Now check to see if the user is on the allowed group list
            
            $query = "SELECT * FROM soybo_access_ip WHERE ip_address='$addr' AND type='2' AND user_group_id='$group_id' AND access='2'";
            $result = $_DATABASE->Query($query);
            
            if($result && ($num = mysql_num_rows($result) > 0)) {
                GFn_Log_Event(kLOG_USER, "Login");
                return array(true, true);
            }
            
                // Now check to see if the user is on the banned group list
            
            $query = "SELECT * FROM soybo_access_ip WHERE ip_address='$addr' AND type='2' AND user_group_id='$group_id' AND access='1'";
            $result = $_DATABASE->Query($query);
            
            if($result && ($num = mysql_num_rows($result) > 0)) {
                $err = $_CONF["locale"]->GetString("errPermission");
                $logErr = "Access not allowed for \"$groupname\" group on IP Address: $addr";
                break;
            }

            if($err) break;
        }
        
        if($err) {
            GFn_Log_Event(kLOG_ERROR, $logErr);
            $errTitle = $_CONF["locale"]->GetString("errLoginFailed");
            return Create_Error($errTitle, $err);       
        }

        GFn_Log_Event(kLOG_USER, "Login");
        
        return array(true, true);
    }




    /* Function:    Get_User_Client_Classname
     * 
     * Description: Retrieves the client's classname from the user client preferences database.
     *              This overrides an entry in the soybo_clients database (since it is a user preference).
     *
     * Parameters:  session_id - the session to check
     *              useragent - the HTTP User Agent of the browser accessing the system
     *
     * Returns:     the class name of the client to instantiate or false
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */

    function Get_User_Client_Classname($session_id, $useragent)
    {
        global $_CONF;
        global $_DATABASE;

            // First get the user of the session
        
        $query = "SELECT * FROM soybo_sessions WHERE session_id='$session_id'";
        $result = $_DATABASE->Query($query);
        
        if($row = mysql_fetch_assoc($result)) {
            $user_id = $row["user_id"];
            
                // Check the user's client preferences to see is there is an override present
            
            if($user_id) {
                $query = "SELECT * FROM soybo_users_clients WHERE user_id='$user_id' AND useragent='$useragent'";
                $result = $_DATABASE->Query($query);
            
                if($row = mysql_fetch_assoc($result)) {
                    $clientname = $row["clientname"];
                    
                        // Return the appropriate client class name
                    
                    return ($clientname) ? $clientname : $this->Get_Client_Classname($useragent);
                    
                } else {
    
                        // Add the useragent to the list user client list
                        // Since we don't fill in a clientname field, the user will receive the default
                        // client associated with this user agent
            
                    $query = "INSERT INTO soybo_users_clients (user_id, useragent) VALUES ('$user_id', '$useragent')";
                    $result = $_DATABASE->Query($query);
                }
            }
        }

        return false;
    }



    /* Function:    Get_Client_Classname
     * 
     * Description: Retrieves the client's classname from the database. If it doesn't
     *              exist, the function adds's the useragent to the database and returns
     *              the default client classname.
     *
     * Parameters:  useragent - the HTTP User Agent of the browser accessing the system
     *
     * Returns:     the class name of the client to instantiate or an error
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */

    function Get_Client_Classname($useragent)
    {
        global $_CONF;
        global $_DATABASE;
        
            // Get the user-agent string
            
        $query = "SELECT * FROM soybo_clients WHERE useragent='$useragent'";
        $result = $_DATABASE->Query($query);
        
        if($result) {
            $row = mysql_fetch_assoc($result);
            
            if($row) {
                $client_classname = ($row["clientname"]) ? $row["clientname"] : $_CONF["default_client_classname"];
            } else {
                GFn_Log_Event(kLOG_SERVER, "Detected new user-agent: $useragent");
                
                $query = "INSERT INTO soybo_clients (useragent) VALUES ('$useragent')";
                $result = @mysql_query($query) or die(mysql_error());
                                
                    // Display the default client
                    
                $client_classname = $_CONF["default_client_classname"];
            }
        } else {
            $client_classname = $_CONF["default_client_classname"];
        }

            // Make sure the client class exists
            
        if(class_exists($client_classname)) {
        
            return $client_classname;
            
        } else {
        
                // If it doesn't exist, try the default classname. If that doesn't
                // exist, error
            
            $client_classname = $_CONF["default_client_classname"];
            
            if(class_exists($clien_classname))
                return $client_classname;
        
                // Uh oh, we should error
        
            $errMsg = "Client Class \"$client_classname\" has not been loaded!";
            
            GFn_Log_Event(kLOG_ERROR, "Client Class \"$client_classname\" has not been loaded!");
            
            die("Unknown client class name.");
        }
    }



    /* Method:      Retrieve_Home
     * 
     * Description: Builds the list of directories and services that the user has access to.
     *
     * Parameters:  username - the user whose home object should be retrieved
     *
     * Returns:     the home object
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */ 
     
    function Retrieve_Home($username)
    {   
        global $_CONF;
        global $configuration;

        
        // the new way of doing things
        
        global $_DATABASE;
        
        $user = $_DATABASE->Get_User($username);
        $user_id = $user["user_id"];
        $group_id = $user["group_id"];
        
        $home = array();
        $home["directories"] = array();
        $home["services"] = array();
        
        // Retrieve the user and group sub-directories
    
        $result = $_DATABASE->Query("SELECT * FROM soybo_directories WHERE type='1' AND user_group_id='$user_id'");
        while($directory = mysql_fetch_assoc($result)) $home["directories"][$directory["dir_id"]] = $directory;
        $result = $_DATABASE->Query("SELECT * FROM soybo_directories WHERE type='2' AND user_group_id='$group_id'");
        while($directory = mysql_fetch_assoc($result)) $home["directories"][$directory["dir_id"]] = $directory;
        
            // Retrieve the user and group services
            
        $result = $_DATABASE->Query("SELECT * FROM soybo_services_users_groups WHERE type='1' AND user_group_id='$user_id'");
        while($service = mysql_fetch_assoc($result)) {
            $service_name = $service["service_name"];
            $result2 = $_DATABASE->Query("SELECT * FROM soybo_services WHERE service_name='$service_name'");
            $service2 = mysql_fetch_assoc($result2);
            
            if($service2) {
                $service = array_merge($service, $service2);
                $home["services"][$service["service_name"]] = $service;
            }
        }
        $result = $_DATABASE->Query("SELECT * FROM soybo_services_users_groups WHERE type='2' AND user_group_id='$group_id'");
        while($service = mysql_fetch_assoc($result)) {
            $service_name = $service["service_name"];
            $result2 = $_DATABASE->Query("SELECT * FROM soybo_services WHERE service_name='$service_name'");
            $service2 = mysql_fetch_assoc($result2);
            
            if($service2) {
                $service = array_merge($service, $service2);
                $home["services"][$service["service_name"]] = $service;
            }
        }
        
        return $home;
    }
    


    /* Method:      Run
     * 
     * Description: Prepares to run the requested script with the given
     *              methodName and parameter lists.
     *
     * Parameters:  methodName - the method to be called in the script
     *              parameters - list of parameters to send to the script
     *
     * Returns:     an array containing the result of the script or an error
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */
     
    function Run($scriptPath, $methodName, $parameters)
    {   
        global $_CONF;
                
            // Execute the script and return the result
            
        if(file_exists($scriptPath)) {
            $path_parts = pathinfo($scriptPath);
            $extension = $path_parts["extension"];
           
            $username = $_CONF["username"];
            $parameters["path_preferences"] = $_CONF["path_users"] . "$username/";
            $parameters["site_url"] = $_CONF["site_url"];

            if(!file_exists($parameters["path_preferences"])) {
                mkdir($parameters["path_preferences"]);
            }

            unset($parameters[kCMD_PARAMETER_SCRIPT]);
            unset($parameters[kCMD_PARAMETER_COMMAND]);
        
            //echo_object(0, $parameters, "Parameters");
        
                // Get the appropriate script handler
        
            if($extension) {
                $handlerName = strtoupper($extension) . "_Script_Handler";

                if(class_exists($handlerName)) {
                    $handler = new $handlerName;
                    
                        // Run the script
                        
                    $result = $handler->RunScript($scriptPath, $methodName, $parameters);
                    
                    return ($result) ? array(true, $result) : Create_Error("", $_CONF["locale"]->GetString("errRun"));
                    
                } else {
                    
                    GFn_Log_Event(kLOG_ERROR, "The script handler does not exist on this server: " . $extension);
                    
                    return Create_Error("404", $_CONF["locale"]->GetString("errScriptHandlerNotFound"));
                }
            }
        } else {
            return Create_Error("404", $_CONF["locale"]->GetString("errFileNotFound"));
        }
    }

    /* Function:    Verify_Service
     * 
     * Description: Verifies that the user can run the given service.
     *
     * Parameters:  username - the username to check
     *              service_name - the unique name for the service
     *
     * Returns:     false on success, error message otherwise
     *
     * Revision History:
     *
     *  2003-03-22 - Initial version (adam)
     *
     *
     */
     
    function Verify_Service($username, $service_name)
    {
        global $_CONF;
        global $_DATABASE;
        
        $locale = $_CONF["locale"]->GetStrings();
        $user = $_DATABASE->Get_User($username);
        
        if(!$user) return $locale["errUserNotFound"];

        $user_id = $user["user_id"];
        $group_id = $user["group_id"];
            
            // Verify that the user has access to the service
    
        $result = $_DATABASE->Query("SELECT * FROM soybo_services_users_groups WHERE type='1' AND user_group_id='$user_id' AND service_name='$service_name'");
        $num = mysql_num_rows($result);
        
        if($num == 0) {
            $result = $_DATABASE->Query("SELECT * FROM soybo_services_users_groups WHERE type='2' AND user_group_id='$group_id' AND service_name='$service_name'");
            $num = mysql_num_rows($result);
            
            if($num == 0)
                return $locale["errServicePermissions"];
        }
    
            // User has access, return false
        
        return false;
    }
    


    /* Method:      Get_Service_Locale
     * 
     * Description: This function retrieves the service locale
     *
     * Parameters:  path_service - the path to the service
     *              service_default_locale - the name of the default service locale
     *
     * Returns:     an array of locale strings or false if no locale was found
     *
     * Revision History:
     *
     *  2003-03-21 - Initial version (adam)
     *
     *
     */

    function Get_Service_Locale($path_service, $service_default_locale)
    {
        global $_CONF;
        
            // Retrieve the current locale name
            
        $localeName = strtolower($_CONF["locale_name"]);
        
            // Retrieve the path to the localization file
        
        $localePath = $path_service . "soybo_localization/$localeName.php";
                        
        if(!file_exists($localePath)) {
            $localeName = strtolower($service_default_locale);
            $localePath = $path_service . "soybo_localization/$localeName.php";
        }
                                    
        if(file_exists($localePath)) {
            
            $serviceLocale = new Locale($localeName, $localePath);
            
            return $serviceLocale->GetStrings();
        }

        return false;
    }


    /* Method:      Get_Service_Templates
     * 
     * Description: This function retrieves a list of the service's templates
     *
     * Parameters:  path_service - the path to the service
     *
     * Returns:     an array of templates filenames
     *
     * Revision History:
     *
     *  2003-03-21 - Initial version (adam)
     *
     *
     */

    function Get_Service_Templates($path_service)
    {
        $templatesPath = $path_service . "templates";
        $templates = array();
        
        if(file_exists($templatesPath) && is_dir($templatesPath)) {
            $templatesDir = dir($templatesPath);
            
            while(false !== ($templateFile = $templatesDir->read())) {
                $templateFilePath = $templatesPath . "/" . $templateFile;
                $templateFileType = filetype($templateFilePath);
                $len = strlen($templateFile);

                if($templateFileType == "file") {
                    $pathParts = pathinfo($templateFilePath);
                    $extension = $pathParts["extension"];
                    $basename = basename($pathParts["basename"], "." . $extension);
                    
                    if($extension == "tpl") {
                        $templates[$basename] = $templateFilePath;
                    }
                }   
            }
        }
        return $templates;
    }



    /* Method:      Session_Generate
     * 
     * Description: This function adds a new session to the session database
     *
     * Parameters:  session_id - the current session id
     *              ip_address - the ip address that the user is accessing the system from
     *              username   - the username accessing the system.
     *
     * Returns:     true on success, false otherwise
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */

    function Session_Generate($session_id, $ip_address, $username)
    {
        global $_CONF;
        global $_DATABASE;
        
        $result = false;
        $user_id = $_DATABASE->Get_User_ID($username);
        
            // Clear out all other sessions with the current user (except guest sessions)
        
        if($username != "guest") {
            $query = "DELETE FROM soybo_sessions WHERE user_id='$user_id'";
            $result = mysql_query($query);

                // Remove any guest session that might be active now
            
            $_DATABASE->Remove_Row("soybo_sessions", "session_id", $session_id);
        } 
                
            // Expire the session if not activity after this time period
        
        $session_start = date("Y-m-d H:i:s", time());
        $session_end = date("Y-m-d H:i:s", time() + $_CONF["session_expires_after"]);
        
            // Get the User-Agent of the browser for the session. We verify this in the Session_Verify method
        
        $useragent = $_SERVER["HTTP_USER_AGENT"];

            // Now create a new session into the database

        $query = "INSERT INTO soybo_sessions (session_id, ip_address, user_id, session_start, session_end, useragent) VALUES ('$session_id','$ip_address', '$user_id', '$session_start', '$session_end', '$useragent')";
        $result = mysql_query($query);
        
        if($result) {
            GFn_Log_Event(kLOG_SESSION, $session_id);
            
            $this->session_id = $session_id;
        }

        return $result;
    }


    /* Method:      Session_Destroy
     * 
     * Description: Logs the user off the system by removing the user's
     *              session from the database 
     *
     * Parameters:  session_id - the session to delete
     *
     * Returns:     true or false on error
     *
     * Revision History: 2003-03-15 - adam
     *
     *
     */

    function Session_Destroy($session_id)
    {
        global $_DATABASE;
        
        GFn_Log_Event(kLOG_USER, "User Logoff");
        
        $result = $_DATABASE->Remove_Row("soybo_sessions", "session_id", $session_id);
        
        return $result;
    }



    /* Method:      Session_Valid
     * 
     * Description: Verifies a session using the IP Address and User-Agent strings, along
     *              with checking the expiration time on the session.
     *
     * Parameters:  session_id - the session to verify
     *
     * Returns:     kSESSION_EXPIRED, kSESSION_VALID, or kSESSION_NONE
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */

    function Session_Valid($session_id)
    {
        global $_CONF;
        global $_DATABASE;
        
            // Clean up any expired sessions
            
        $this->Session_Cleanup();
        
        $query = "SELECT * FROM soybo_sessions WHERE session_id='$session_id'";
        $result = $_DATABASE->Query($query);
        $numRows = mysql_num_rows($result);

        if($numRows == 0)
            return kSESSION_NONE;
            
        if($numRows == 1) {
            $currentTime = time();
            $row = mysql_fetch_assoc($result);
            
            $expireTimeMySQL = $row["session_end"]; // Get the expiration time
            $session_id_address = $row["ip_address"];
            
            $session_start = $row["session_start"];
            
            $query = "SELECT UNIX_TIMESTAMP('$expireTimeMySQL')";
            $result = $_DATABASE->Query($query);
            $row2 = mysql_fetch_row($result);
            $expirationTime = $row2[0];
            
            
                // We should ensure that the IP Address in which the session is coming from is the
                // same as what the remote computer says. Otherwise, someone might be spoofing the Session ID
                
            $ip_address = $_SERVER["REMOTE_ADDR"];
            
                // Check the useragent of the request
                
            $useragent = $_SERVER["HTTP_USER_AGENT"];
            $sessionUserAgent = $row["useragent"];
            
            if(($ip_address == $session_id_address && $useragent == $sessionUserAgent) || ($_CONF["allow_loopback_calls"] && $ip_address == "127.0.0.1")) {

                if($currentTime > $expirationTime) {
                    $this->Session_Destroy($session_id);
                    
                    return kSESSION_EXPIRED;
                    
                } else {
                    
                        // Now we add some more time to the session
                        
                    $newExpireTime = date("Y-m-d H:i:s", time() + $_CONF["session_expires_after"]);
                    
                    $query = "UPDATE soybo_sessions SET session_end='$newExpireTime', session_start='$session_start' WHERE session_id='$session_id'";
                    
                        // Set the username in the global configuration
                    
                    $_CONF["username"] = $_DATABASE->Get_Username($row["user_id"]);
                    
                        //echo "Session Validation retrieved username: " . $_CONF["username"]  . "<br />";
                    
                    $result = $_DATABASE->Query($query);
                }
                
                return kSESSION_VALID;
            } else {                
                if($ip_address != $session_id_address)
                    GFn_Log_Event(kLOG_ERROR, "Session IP Address ($session_id_address) does not match requesting IP Address ($ip_address)!");
                else GFn_Log_Event(kLOG_ERROR, "Client ($sessionUserAgent) does not match requesting client ($useragent)!");

                return kSESSION_INVALID;
            }
        } else {
            die("Multiple session id's! This should never happen");
        }
        
        return kSESSION_INVALID;
    }




    /* Method:      Session_Cleanup
     * 
     * Description: Removes any any expired sessions
     *
     * Parameters:  None
     *
     * Returns:     true
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */
     
    function Session_Cleanup()
    {   
        global $_CONF;
        global $_DATABASE;
        
        $time = time();
        
        $query = "SELECT session_cleanup_time FROM soybo_preferences WHERE id='1'";
        $result = $_DATABASE->Query($query);
        
        if($result) {
            $row = mysql_fetch_assoc($result);
            $lastCleanup = $row["session_cleanup_time"];    
            $query = "SELECT UNIX_TIMESTAMP('$lastCleanup')";
            $result = mysql_query($query);
            $row = mysql_fetch_row($result);
            $lastCleanup = $row[0];
            $diff = time() - $lastCleanup;

                // Check to see if it's time to cleanup dangling sessions
                
            if($diff > $_CONF["session_cleanup_delay"]) {
            
                    // Delete all sessions whose times have expired
                    
                $currentTime = date("YmdHis", time());
                $query = "DELETE FROM soybo_sessions WHERE session_end <= '$currentTime'";
                $result = mysql_query($query);
                
                if($result) {
                    $numRows = mysql_affected_rows();
                    
                        // Reset the last session cleanup time to now
                        
                    if($numRows > 0) {
                        $newCleanupTime = date("Y-m-d H:i:s", $time);
                        $query = "UPDATE soybo_preferences SET session_cleanup_time='$newCleanupTime' WHERE id='1'";
                        $result = $_DATABASE->Query($query);
                            
                            // Log the session cleanup action
                        
                        GFn_Log_Event(kLOG_SERVER, "Session Cleanup: $numRows sessions were expired.");
                    }
                }
            }
        }   
        
        return true;
    }
}




/* Function:    _Load_One_File()
 * 
 * Description: Loads the PHP files contained in the server directory. Only searches
 *              for PHP files.
 *
 * Parameters:  dirPath - the directory to search
 *
 * Returns:     true
 *
 * Revision History:
 *
 *  2003-03-15 - Initial version (adam)
 *
 */

function _Load_One_File($dirPath)
{
    $dir = dir($dirPath);
    
    while(false !== ($file = $dir->read())) {
        $pathInfo = pathinfo($file);

            // Only include php files
            
        if($pathInfo["extension"] == "php") {       
            $filePath = $dirPath . "/" . $file;
            
            include($filePath);
        }
    }
    $dir->close();
    
    return true;
}


/* Function:    GFn_Load_Custom_Servers()
 * 
 * Description: Scans through the servers directory for new servers to install
 *
 * Parameters:  None
 *
 * Returns:     true
 *
 * Revision History:
 *
 *  2003-03-15 - Initial version (adam)
 *
 */

function GFn_Load_Custom_Servers()
{       
    global $_CONF;
    
    $pathToCustomDevices = $_CONF["path_html"] . "/support/servers/";
    $dir = dir($pathToCustomDevices);

    while(false !== ($file = $dir->read())) {
        $filepath = $pathToCustomDevices . $file;
        $filetype = filetype($filepath);
        
        if(substr($file, 0, 1) != "." && $filetype == "dir") {
            _Load_One_File($filepath);
        }   
    }
    $dir->close();
    
    return true;
}

?>
