<?php

/* Copyright (C) 2003 Adam Tow

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
   
   
/* Class:       Workspace
 * 
 * Description: Class definition for the object that will contain a user's
 *              current workspace, including:
 *
 *              username - the username
 *              directory - the directory the user is currently in
 *              script - the script the user was last running
 *              home - the home object
 *              page - the last page object the user was in
 *
 * Revision History:
 *  
 *  2003-03-17 - Initial version (adam)
 *
 *
 */
 
class Workspace
{
    function Workspace()
    {
        $this->info = array();
    }



    /* Method:      Set
     *
     * Description: Sets a value in the workspace info array.
     *
     * Parameters:  name - the name of the element.
     *              value - the value of the element.
     *
     * Returns:     Nothing
     *
     * Revision History:
     *
     *  2003-03-17 - Initial version (adam)
     *
     *
     */
     
    function Set($name, $value)
    {
        $this->info[$name] = $value;
    }


    
    /* Method:      Get
     *
     * Description: Retrieves a value from an element in the workspace info array
     *
     * Parameters:  name - the name of the element to retrieve
     *
     * Returns:     the value of the element
     *
     * Revision History:
     *
     *  2003-03-17 - Initial version (adam)
     *
     *
     */
     
    function Get($name)
    {
        return $this->info[$name];
    }
    
    
    
    /* Method:      Save
     *
     * Description: Saves the workspace to the database. We take the workspace object and base64
     *              encode it. The resulting string can then be placed into the soybo_workspaces table,
     *              referenced by the user's user id.
     *
     * Parameters:  None
     *
     * Returns:     true or an error
     *
     * Revision History:
     *
     *  2003-03-17 - Initial version (adam)
     *
     *
     */
     
    function Save()
    {
        global $_DATABASE;
        
        $username = $this->Get("username");
        
            // Serialize the info array
        
        $serializedInfo = serialize($this->info);
        $slashedSerializedInfo = base64_encode($serializedInfo);
                
            // Insert the serialized workspace into the soybo_workspaces table
        
        $user_id = $_DATABASE->Get_User_ID($username);

        if($user_id) {
            $result = $_DATABASE->Get_Row("soybo_workspaces", "workspace", "user_id", $user_id);
            
            if($result) {               
                $result = $_DATABASE->Update_Row("soybo_workspaces", "workspace", $slashedSerializedInfo, "user_id", $user_id);
            } else {
                $query = "INSERT INTO soybo_workspaces (user_id, workspace) VALUES ('$user_id','$slashedSerializedInfo')";
                $result = mysql_query($query);
            }   
            
            return $result;

        } else {
            die("An error occurred while saving the user's workspace. User was not found in the database.");
        }
        
        return true;
    }

    
    /* Method:      Retrieve
     *
     * Description: Retrieves a user's workspace from the soybo_workspaces database table. We decode
     *              the base64 string to rehydrate the workspace object.
     *
     * Parameters:  username - the user whose workspace should be retrieve
     *
     * Returns:     Workspace object or false if no workspace was found. 
     *
     * Revision History:
     *
     *  2003-03-17 - Initial version (adam)
     *
     *
     */
     
    function Retrieve($username)
    {   
        global $_DATABASE;
                    
        //echo "Retrieving workspace for: $username<br />";
        
            // Access the database and retrieve the user's current workspace
        
        $user_id = $_DATABASE->Get_User_ID($username);
        
        if($user_id) {
        
            $result = $_DATABASE->Get_Row("soybo_workspaces", "workspace", "user_id", $user_id);
            
            if($result) {
                $workspace = $result["workspace"];
                
                $this->info = unserialize(base64_decode($workspace));
                
                return true;
            
            } else {
                $this->info = array();
                
                return false;
            }
        } else {
            die("An error occurred while retrieving the user's workspace. User was not found in the database.");
        }
    }
}

?>