<?php

/* Copyright (C) 2003 Adam Tow

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
   


/* Class:       HTTP_Client
 *
 * Extends:     Client
 *
 * Description: The default class for clients that communicate with
 *              the Soybo server using HTTP.
 *
 * Revision History:
 *
 *  2003-03-15 - Initial version (adam)
 *
 */

class HTTP_Client extends Client
{
    var $page;
        
        // Class Paths (overriden by subclasses)
        
    var $mediaPath;
    var $cssPath;
    var $templatesPath;
    
        // Inherited Path Arrays (added to by subclasses)
    
    var $mediaPaths;
    var $cssPaths;
    var $templatesPaths;
    
    function HTTP_Client()
    {
        global $_CONF;
        global $_PAGE;
        
        $this->Client();
        
            // Set up my media, stylesheet, and template paths

        $this->mediaPath = "/skins/http";
        $this->cssPath = $this->mediaPath . "/stylesheets/default.css";
        $this->templatesPath = $_CONF["path_html"] . "/skins/http/templates/";
        
            // Set up the inherited paths array
        
        $this->cssPaths = array($this->cssPath);
        $this->mediaPaths = array($this->mediaPath);
        $this->templatesPaths = array($this->templatesPath);

            // Initialize Smarty and Workspace
            
        $this->page = &$_PAGE;
    }
    
    
    
    /* Method:      Main
     *
     * Description: Entry point for the client class. Determines what page to display
     *              or what function to run.
     *
     * Parameters:  session_id - the current session id
     *              session_state - the state of the session: valid, none, expired, etc.
     *
     * Returns:     true
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     */
     
    function Main($session_id, $session_state)
    {
        global $_CONF;
        
        $cmd = $_REQUEST[kCMD_PARAMETER_COMMAND];
        
        if(!$session_id) {

                // Though we create the session_id now,
                // We won't store it into the database until the
                // user actually logs in.
                
            $_SESSION["session_id"] = session_id();
            
                // Display the login page or log in as guest
            
            if(($cmd && $cmd == kCMD_LOGIN_PAGE) || !$_CONF["allow_guest_access"]) {
                $this->Login_Display();
            } else if($_CONF["allow_guest_access"]) {
                $guestAuthenticationFrame = array("username" => "guest");
                $this->Login($guestAuthenticationFrame);
            } else {
                $this->Login_Display();
            }
        } else {

            switch($session_state)
            {
                case kSESSION_VALID:    $this->_Main();
                                        break;

                case kSESSION_EXPIRED:  $cmd = $_REQUEST[kCMD_PARAMETER_COMMAND];
                    
                                        if($cmd && $cmd == kCMD_LOGIN)
                                            $this->Login($_REQUEST);
                                        else
                                            $this->Session_Expired_Display();
                                        break;
            
                case kSESSION_NONE:     
                default:                $cmd = $_REQUEST[kCMD_PARAMETER_COMMAND];
                                        
                                        if($cmd && $cmd == kCMD_LOGIN) {
                                            $this->Login($_REQUEST);
                                        } else if($cmd && $cmd == kCMD_LOGIN_PAGE) {
                                            $this->Login_Display();
                                        } else if($_CONF["allow_guest_access"]) {
                                            $guestAuthenticationFrame = array("username" => "guest");
                                            $this->Login($guestAuthenticationFrame);
                                        } else {
                                            $this->Login_Display();
                                        }
                                        break;
            }
        }
        
        return true;
    }



    /* Method:      _Main
     *
     * Description: Switchboard for Soybo server command
     *
     * Parameters:  None
     *
     * Returns:     true
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */
     
    function _Main()
    {   
        global $_CONF;
        
        $cmd = $_REQUEST[kCMD_PARAMETER_COMMAND];

        //echo_object(0, $_REQUEST, "Request parameters");
        
            // Retrieve the user's workspace
        
        $this->Workspace_Get();
        
        switch($cmd)
        {
            case kCMD_LOGIN:        $this->Login($_REQUEST);
                                    break;
                                
            case kCMD_RUN:          $this->Run($_REQUEST);                          
                                    $this->Workspace_Save(false);
                                    break;

            case kCMD_CD:           $this->Directory_Change($_REQUEST[kCMD_PARAMETER_PATH]);
                                    $this->Workspace_Save(false);
                                    break;

            case kCMD_LOGOFF:       $this->Logoff();
                                    break;
        
            case kCMD_LOGIN_PAGE:   $this->Login_Display();
                                    break;
            
            case kCMD_REFRESH:      $this->home = $this->server->Retrieve_Home($_CONF["username"]);
                                    $this->Directory_Change(false);
                                    $this->Workspace_Save(false);
                                    break;
                        
            default:                $this->Directory_Change(false);
                                    $this->Workspace_Save(false);
                                    break;
        }
        
        return true;
    }



    /* Method:      Login
     *
     * Description: Attempts to log the user into the server
     *
     * Parameters:  authenticationFrame - array containing authentication information
     *
     * Returns:     true or error page
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     */
     
    function Login($authenticationFrame)
    {   
        global $_CONF;
        
        $_CONF["username"] = $authenticationFrame["username"];

        list($success, $response) = $this->server->Login($authenticationFrame);

        if($success) {
            $this->home = $response;
            $locale = strtolower($authenticationFrame["locale"]);
            $workspaceAction = strtolower($authenticationFrame["workspace"]);
            
                // Retrieve the user's workspace
                
            $result = $this->workspace->Retrieve($authenticationFrame["username"]);
            
                // If the workspace is valid, set information for the last page the user was at
            
            if($result && (!$workspaceAction || $workspaceAction == "resume")) {
                $this->workspace->Set("home", $response);
                $this->current_directory = $this->workspace->Get("current_directory");
                $this->current_service = $this->workspace->Get("current_service");
            } else {

                    // No workspace was found in the database

                $this->current_directory = "";
                $this->current_service = "";
            }
            
                // Set the locale
                
            if($locale && $locale == "default") {
                $_CONF["locale"] = GFn_Get_Locale($this->workspace->Get("locale"));
            } else {
                $this->workspace->Set("locale", $locale);
                $_CONF["locale"] = GFn_Get_Locale($locale);
            }
            
                // Display the last page for members of the system, not guests
            
            if($_CONF["username"] != guest) {
                if($workspaceAction != "new") {
                    if($currentResponse = $this->workspace->Get("currentResponse")) {
                        $this->Handle_Response($currentResponse);
                    } else {
                        $this->Directory_Change($this->current_directory);   
                    }
                } else {
                    $this->Directory_Change($this->current_directory);   
                }
            } else {
                $this->Directory_Change(false);
            }
                    
                // Save the workspace
            
            $this->Workspace_Save(true);
            
        } else {
        
                // If Guests can't log in, it's because access is not permitted from that domain.
                // So, we display the login page. Otherwise, we display the error page.
                
            if($_CONF["username"] == "guest") {
                $this->Login_Display();
            } else {
                $this->Error_Server($response);
            }
        }
        
        return true;
    }



    /* Method:      Logoff
     *
     * Description: Logs the user off the system and clears the current session variables
     *
     * Parameters:  None
     *
     * Returns:     true
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */
     
    function Logoff()
    {
        $session_id = $_SESSION["session_id"];      
        
        $this->server->Session_Destroy($session_id);
        
        session_destroy();

        $this->Logoff_Display();
        
        return true;
    }
    
    

    /* Method:      Logoff_Display
     *
     * Description: Displays the logoff page
     *
     * Parameters:  None
     *
     * Returns:     true
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */
            
    function Logoff_Display()
    {       
        $this->Template_Display("logoff", true);
        
        return true;
    }



    /* Method:      Session_Expired_Display
     *
     * Description: Displays the session expired page
     *
     * Parameters:  None
     *
     * Returns:     true
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */

    function Session_Expired_Display()
    {       
        $title = $this->page->PageGet("localization", "sessionExpiredTitle");   
        $this->page->PageSet("title", $title);
        $this->Template_Display("session_expired", true);
        
        return true;
    }



    /* Method:      Templates_Generate
     *
     * Description: Populates the array of templates for a given client class. These
     *              templates will be available in the Smarty templates under the
     *              $_SERVER.templates variable. To include on these files,
     *              enter the template name minus the file extension.
     *
     *              (i.e.) $_SERVER.templates.directory to access the directory.tpl template file
     *
     * Parameters:  None
     *
     * Returns:     templates array
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */

    function Templates_Generate()
    {
        $templates = array();
        $len = count($this->templatesPaths) - 1;

        for($i = $len; $i >= 0; $i--) {
            $templatePath = $this->templatesPaths[$i];

            if(file_exists($templatePath)) {
                $handle = opendir($templatePath);
        
                while(false !== ($file = readdir($handle))) {
                    if(substr($file, 0, 1) != ".") {
                        $pathParts = pathinfo($templatePath . $file);
                        $basename = basename($pathParts["basename"], "." . $pathParts["extension"]);
                        
                        if(!$templates[$basename])
                            $templates[$basename] = $templatePath . $file;
                    }
                }
                closedir($handle);
            }
        }
        return $templates;
    }   



    /* Method:      Template_Display
     *
     * Description: Displays a page to the user. This function populates
     *              the template with a variety of variables that will aid in
     *              creation of template files.
     *
     * Parameters:  templatePath - the path to the template
     *              isClientTemplate - specifies whether the template is a client class template or a Soyvice template
     *
     * Returns: true
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     */
        
    function Template_Display($templatePath, $isClientTemplate)
    {
        global $_CONF;
        global $timer;
        
            // Generate the Navigation Bar and Templates
        
        $navbarURIs = $this->Navbar_Generate();
        $templates = $this->Templates_Generate();
    
            // Set the version of the Soybo Server
                
        $server["version"]                  = $_CONF["version"];
        
            // Set the URI's to the media directories
        
        $server["dir"]["media"]             = $_CONF["uri_media"];
        $server["dir"]["media_services"]    = $this->Media_Path_Generate("", false);
        $server["dir"]["icons"]             = $_CONF["uri_folder_icons"];
        
        $server["navbar"]                   = $navbarURIs;
        
            // Set the URL to the server
        
        $server["url"]                      = $_CONF["uri"] . "/";
        
            // Set the server command variables depending
        
        $server["pages"]["help"]            = $_CONF["url_help"];
        $server["pages"]["login"]           = $_CONF["uri_server"] . "?c=" . kCMD_LOGIN_PAGE;
        
        $server["cmd"]["login"]             = $_CONF["uri_server"] . "?c=" . kCMD_LOGIN;
        $server["cmd"]["service"]           = $_CONF["uri_server"] . "?c=" . kCMD_RUN . "&amp;s=";
        $server["cmd"]["directory"]         = $_CONF["uri_server"] . "?c=" . kCMD_CD . "&amp;p=";
        $server["cmd"]["refresh"]           = $_CONF["uri_server"] . "?c=" . kCMD_REFRESH;
        $server["cmd"]["logoff"]            = $_CONF["uri_server"] . "?c=" . kCMD_LOGOFF;

        $server["cmd"]["service_raw"]       = $_CONF["uri_server"] . "?c=" . kCMD_RUN . "&amp;s=";      
        $server["cmd"]["directory_raw"]     = $_CONF["uri_server"] . "?c=" . kCMD_CD . "&amp;p=";
        
            // Get a list of all images for the client
            
        $server["images"]                   = $this->Media_Images_Generate();       
        $server["user"]["username"]         = $_CONF["username"];
        $server["locale"]                   = $_CONF["locale"]->GetStrings();
        $server["templates"]                = $templates;
        
        foreach($this->cssPaths as $cssPath) {
            $cssPath = $_CONF["uri"] . $cssPath;
            $this->page->PageAdd("head", "<link rel=\"stylesheet\" href=\"$cssPath\" />");
        }
        
            // Set the stylesheets for the client
        
        $server["stylesheets"] = $this->page->PageGet("head");
        
        $this->page->assign("title", $this->page->PageGet("title"));

            // Assign the _CLIENT variable to the template

        $client["type"] = $this->type;
        $client["name"] = $this->name;
        $client["user-agent"] = $_SERVER["HTTP_USER_AGENT"];
        $this->page->assign("_CLIENT", $client);
        
            // Set the time variable. This is the time elapsed for script execution
        
        $time = $timer->endTimer(false);
        $server["time"]["script"] = $time;

            // Assign the _SERVER variable to the template
                
        $this->page->assign("_SERVER", $server);
        
            // Display the template
                
        $theTemplate = ($isClientTemplate) ? $templates[$templatePath] : $templatePath; 
        $this->page->display($theTemplate);
    }



    /* Method:      Navbar_Generate
     *
     * Description: Generate the paths that will be displayed in the navigation bar on
     *              all pages.
     *
     * Parameters:  None
     *
     * Returns:     The array of navbar uri's
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */
    
    function Navbar_Generate()
    {
        global $_CONF;

        $navbarPaths = array();
        
            // Get the name of the service
            
        if($this->current_service) {
            $service_name = $this->current_service;
            $service = $this->home["services"][$service_name];
            $name = $service["name"];
            
            array_push($navbarPaths, array("path" => $service_name, "cmd" => "run", "name" => $name));
        }
        
            // Create the directory paths
        
        $dir_id = $this->current_directory;
        
        while($dir_id != 0) {
            $directory = $this->home["directories"][$dir_id];
            
            array_push($navbarPaths, array("path" => $dir_id, "cmd" => "cd", "name" => $directory["dirname"]));
            
            $dir_id = $directory["dir_parent"];
        }
        
            // Add the home link
        
        array_push($navbarPaths, array("path" => 0, "cmd" => "cd", "name" => $_CONF["locale"]->GetString("labelHome")));
              
        return array_reverse($navbarPaths);
    }

        
    /* Method:      Directory_Change
     *
     * Description: Function to change directories
     *
     * Parameters:  dir_id - the directory id of the directory to display
     *
     * Returns:     true
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     */
     
    function Directory_Change($dir_id)
    {       
        global $_CONF;
        global $_DATABASE;

        if($dir_id) {
            $directory = $this->home["directories"][$dir_id];
        
            if($directory) {
                $this->workspace->Set("currentResponse", false);
                $this->Directory_Display($directory);
            } else {
                $this->Error_Server(array("faultCode" => "404", "faultString" => $_CONF["locale"]->GetString("errFileNotFound")));
            }
        } else {
            $this->workspace->Set("currentResponse", false);
            $this->Directory_Display(false);
        }
        
        return true;
    }





    /* Method:      Directory_Display
     *
     * Description: Displays a directory page to the user.
     *
     * Parameters:  dirPath - the directory path to display
     *              dirObj - array containing the directory's services and sub-directories info
     *
     * Returns:     true
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */
     
    function Directory_Display($directory)
    {   
        global $_CONF;
        global $_DATABASE;
        
        $directories = array();
        $services = array();
        
            // Home Directory for a user. We retrieve the user and group services and directories
        
        if(!$directory) {
            $directory["icon"] = "home.gif";
            $directory["dirname"] = $_CONF["locale"]->GetString("labelHome");
            $directory["description"] = $_CONF["locale"]->GetString("labelHomeDescription");
            $directory["dir_id"] = 0;
            $directory["show_description"] = 1;
        } else {
            $directory["show_description"] = 1;
        }
        
            // We retrieve a list of all sub-directories for the given directory
        
        foreach($this->home["directories"] as $subdir) {
            if($subdir["dir_parent"] == $directory["dir_id"]) {
                $directories[] = $subdir;
            }
        }
        usort($directories, "Cmp_Dirname");
        
            // We retrieve a list of all services for the given directory
        
        foreach($this->home["services"] as $service) {
            if($service["dir_id"] == $directory["dir_id"]) {
                $path_service = $_CONF["path_services"] . $service["path"] . "/";
                
                    // Generate the service's media directory if necessary
                
                $this->Generate_Service_Media($service["service_name"], $path_service);
                
                $services[] = $service;
            }
        }   
        usort($services, "Cmp_Servicename");
  
        $this->current_directory = $directory["dir_id"];
        $this->current_service = false;
        
        $directory["directories"] = $directories;
        $directory["services"] = $services;
                
        $this->page->assign("_DIRECTORY", $directory);
        $this->Template_Display("directory", true);
 
        return true;
    }
    


    /* Method:      Run
     *
     * Description: Runs a service
     *
     * Parameters:  params - the parameter list from the client (i.e. web page form)
     *
     * Returns:     true
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */
    
    function Run($params)
    {           
        global $_DATABASE;
        global $_CONF;
        
        $username = $_CONF["username"];
        
            // Retrieve the service and method names

        $serviceParts = explode(".", strtolower($params[kCMD_PARAMETER_SCRIPT]));
        $methodName = ($serviceParts[1]) ? $serviceParts[1] : "";
        $service_name = ($serviceParts[0]) ? $serviceParts[0] : $service_name;
        $service = &$this->home["services"][$service_name];
        
            // Make sure that the service exists and that the user
            // has permission to run the service        
        
        if(!$service || $err = $this->server->Verify_Service($username, $service_name))
            return $this->Error_Server(array("faultCode" => 404, "faultString" => $_CONF["locale"]->GetString("errServiceNotFound") . $service_name));
        
            // Get the service and script paths
        
        $path_service = $_CONF["path_services"] . $service["path"] . "/";
        $path_script = $path_service . $service["script"];
            
            // Create the service's media directory if necessary
        
        $this->Generate_Service_Media($service_name, $path_service);
    
            // Retrieve the service's locale if necessary
            
        if(!$service["locale"]) {
            $service["locale"] = $this->server->Get_Service_Locale($path_service, $service["default_locale"]);
        }
        
        $params["locale"] = $service["locale"];
        
            // Place references to the media and service paths into the parameters
        
        $params["path_media"] = $this->Media_Path_Generate($service_name, true) . "/";
        $params["path_service"] = $path_service;
    
            // Tell the server to execute the script with the given parameters
    
        list($success, $response) = $this->server->Run($path_script, $methodName, $params);

        if($success) {
    
                // Log the event
            
            if($methodName)
                GFn_Log_Event(kLOG_USER, "$service_name.$methodName");
            else GFn_Log_Event(kLOG_USER, "$service_name");
        
                // Save the response object in the workspace

            $this->current_directory = $service["dir_id"];
            $this->current_service = $service_name;     
            $this->workspace->Set("currentResponse", $response);
            
                // Display the response page
            
            $this->Handle_Response($response);

        } else {
            $this->Error_Server($response);
        }
    
        return true;
     }



    /* Method:      HandleResponse
     *
     * Description: Handle the response from the service's script. Currently, scripts
     *              must response back to the server with a valid XML-RPC response or error document.
     *
     *                  http://www.xmlrpc.com/spec/
     *
     *              If the response is not an XML document, we simply echo the string to the display
     *
     * Parameters:  response - the string returned from the script.
     *
     * Returns:     true or false on error
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     */
     
    function Handle_Response($response)
    {   
        global $_CONF;
        
        $timer = new Timer; 
        $service_name = $this->current_service;
        
            // Check to see if the first line is our response tag
        
        $pos = strpos($response, "<?xml version");
        $pos2 = strpos($response, "<methodResponse>");
            
        if($pos !== false && $pos == 0 && $pos2) {
            $xmlrpc_request = XMLRPC_parse($response);
            
            if(isset($xmlrpc_request['methodResponse']['fault'])){
                $error = XMLRPC_adjustValue(&$xmlrpc_request['methodResponse']['fault']['value']);
                $this->Error_Server($error);
                
                return false;
                
            } else {
                $scriptResponse = XMLRPC_adjustValue(&$xmlrpc_request['methodResponse']['params']['param']['value']);
                
                $serviceObj = array();
                $serviceObj["response"] = $scriptResponse;
                $serviceObj["media"] = $this->Media_Path_Generate($service_name, false);
                
                $service = &$this->home["services"][$service_name];
                $servicePath = $_CONF["path_services"] . $service["path"] . "/";
                
                    // Find the service template
                
                $templateName = $scriptResponse["template"];
                $template = $servicePath . "/templates/$templateName";
                
                if(file_exists($template)) {

                        // Set the script url
                                    
                    $serviceObj["url"] = $_CONF["uri_server"] . "?c=2&amp;s=" . $service_name;
                    
                        // Load the service locale for the service object
                       
                    if(!$service["locale"]) {
                        $service["locale"] = $serviceObj["locale"] = $this->server->Get_Service_Locale($servicePath, $service["default_locale"]);
                    }
                    $serviceObj["locale"] = $service["locale"];
                    
                    if(!$service["templates"]) {
                        $service["templates"] = $this->server->Get_Service_Templates($servicePath);
                    }
                    $serviceObj["templates"] = $service["templates"];
                    
                   
                    $serviceObj["info"] = $service;
                    
                        // Give the template a copy of the request variables
                    
                    $serviceObj["request"] = $_REQUEST;
                                        
                        // Assign the _SERVICE variable to the page template and display the template
                
                    $this->page->assign("_SERVICE", $serviceObj);                   
                    $this->Template_Display($template, false);

                } else {
                    $this->Error_Server(array("faultCode" => 404, "faultString" => $_CONF["locale"]->GetString("errTemplate") . "\"$templateName\""));
                }
            }
        } else {
            if(is_string($response))
                echo $response;
        }
    }

    

    /* Method:      Error_Server
     *
     * Description: Displays an error page to the user
     *
     * Parameters:  fault - array containing the faultCode and faultString to display
     *
     * Returns:     true
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */
    
    function Error_Server($fault)
    {   
        if($fault["faultcode"])
            $faultCode = $fault["faultcode"];
        else $faultCode = $fault["faultCode"];
        
        if($fault["faultstring"])
            $faultString = $fault["faultstring"];
        else $faultString = $fault["faultString"];
        
        $this->page->assign("faultCode", $faultCode);
        $this->page->assign("faultString", $faultString);
        
        $this->Template_Display("error", true);
        
        return true;
    }
    
    

    /* Method:      Workspace_Get
     *
     * Description: Retrieves the user's current workspace
     *
     * Parameters:  None
     *
     * Returns:     true
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */
        
    function Workspace_Get()
    {
        global $_CONF;
        
        $this->workspace->Retrieve($_CONF["username"]);
   
        $this->current_directory = $this->workspace->Get("dir_id");
        $this->current_service = $this->workspace->Get("service_name");
        $this->home = $this->workspace->Get("home");

        $_CONF["locale"] = GFn_Get_Locale($this->workspace->Get("locale"));
        
        return true;
    }


    /* Method:      Workspace_Save
     *
     * Description: Saves the user's current workspace into the database
     *
     * Parameters:  saveGuest - if a guest is logged in, we only save the workspace once when they login
     *
     * Returns:     true
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */
     
    function Workspace_Save($saveGuest)
    {
        global $_CONF;
        
        $username = $_CONF["username"];

        if($username != "guest" || ($username == "guest" && $saveGuest)) {
            $locale = $_CONF["locale"];
    
            $this->workspace->Set("username", $_CONF["username"]);
            $this->workspace->Set("current_directory", $this->current_directory);
            $this->workspace->Set("current_service", $this->current_service);

            $this->workspace->Set("locale", $locale->name);
            $this->workspace->Set("home", $this->home);
            $this->workspace->Set("page", false);
            
            $this->workspace->Save();
        }
                
        return true;
    }
}


?>
