<?php

/* Copyright (C) 2003 Adam Tow

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
   

global $_CLIENTS;

$_CLIENTS["WAP_Client"] = "WAP";


/* Class:       WAP_Client
 *
 * Extends:     HTTP_Client
 *
 * Description: WAP class for Soybo client class. Pages for WAP clients
 *              should be formatted as proper WML documents.
 *
 * Revision History:
 *
 *  2003-03-15 - Initial version (adam)
 *
 *
 */

class WAP_Client extends HTTP_Client
{
    var $mediaPath;
    var $cssPath;
    var $templatesPath;
    
    function WAP_Client()
    {
        global $_CONF;
        
        $this->HTTP_Client();
        
        $this->name = "WAP";
        $this->type = "WAP";

        $this->mediaPath = "/skins/wap";
        $this->templatesPath = $_CONF["path_html"] . "/skins/wap/templates/";
        
        array_push($this->mediaPaths, $this->mediaPath);
        array_push($this->templatesPaths, $this->templatesPath);
        
            // Set the content-type of the response page for WML pages
        
        header("Content-Type: text/vnd.wap.wml");
    }




    /* Method:      Login_Display
     * 
     * Description: Displays a simple login page.
     *
     * Parameters:  None
     *
     * Returns:     Nothing
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */
    
    function Login_Display()
    {
            // Assign the client name and the server path (in the login form)

        $this->page->assign("name", $this->name);

            // Disable the navbar and footer

        $title = $this->page->PageGet("localization", "login");
        $this->page->PageSet("title", $title);      

            // Display the page
            
        $this->Template_Display("login", true);
        
        return true;
    }
    
    
    
    /* Method:      Login
     *
     * Description: This method differs from the one in the HTTP class in that
     *              we always create a fresh new workspace. We are not guaranteed
     *              that the last place the user went to was a WAP-compatible page.
     *
     * Parameters:  authenticationFrame - Authentication frame
     *
     * Returns:     result of inherited method
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */
     
    function Login($authenticationFrame)
    {
            // Always create a new workspace since we are not guaranteed that
            // services will have WAP-compatible templates
        
        $authenticationFrame["workspace"] = "new";
        
            // Call the inherited method
            
        return HTTP_CLIENT::Login($authenticationFrame);
    }
    
    
    
    /* Method:      Template_Display
     *
     * Description: This method differs from the method in the HTTP client class in
     *              that we first check to see if there is a template with the "wap_" prefix
     *              in front of it. If so, we use that template for display. Otherwise, 
     *              we will assume that the template has a client detect built in for
     *              http clients and wap clients.
     *
     * Parameters:  templatePath - The original template path
     *              isClientTemplate - Whether or not this is a client template (as opposed to a service template)
     *
     * Returns:     result of inherited method
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */
    
    function Template_Display($templatePath, $isClientTemplate)
    {
        if(!$isClientTemplate) {
            $pathParts = pathinfo($templatePath);
            $dirname = $pathParts["dirname"];
            $basename = $pathParts["basename"];
            
            $tmpName = "wap_" . $basename;
            $tmpPath = $dirname . "/" . $tmpName;
            
            if(file_exists($tmpPath)) {
                $templatePath = $tmpPath;
            }
        }
        
        return HTTP_CLIENT::Template_Display($templatePath, $isClientTemplate);
    }
    
    
    
    /* Method:      Directory_Generate
     *
     * Description: This method differs from the HTTP client class in that we only display those
     *              services that are marked as being WAP Compatible.
     *
     * Parameters:  name - The name of the directory
     *              path - The path to the directory
     *              dirObj - The directory object
     *
     * Returns:     array containing the directory's sub-directories and services info
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     */
     
    function Directory_Generate($name, $path, $dirObj)
    {
        global $_CONF;
        
        $numScripts = sizeof($dirObj[kSTR_SERVICES]);
        $numDirectories = sizeof($dirObj[kSTR_DIRECTORIES]);
        
        $directoriesArray = array();
        $scriptsArray = array();
        
        if($path == "" || $path == "/") {
            $this->Media_Directory_Generate("directory", "Home", $_CONF["path_users"] . $_CONF["username"] . "/" . kSTR_SERVICES . "/");
        }

            // Only show those services that are marked as WAP Compatible from the Soybo XML Configuration File
                        
        if($numDirectories > 0 || $numScripts > 0) {
            if($numDirectories > 0) {
                foreach($dirObj["directories"] as $childDirectoryName => $childDirectory) {
                    $directory = $this->home["allDirectories"][$childDirectoryName];                                        
                    $childDirectoryPath = $childDirectoryName;
                    
                        // Create the media directory for the directory
                    
                    $this->Media_Directory_Generate("directory", $childDirectoryName, $directory["directory_path"]);
                    
                        // Create the directory path and name for the directory template.
                        
                    if($path[strlen($path) - 1] == "/")
                        array_push($directoriesArray, array("path" => "$path/" . urlencode($childDirectoryPath) . "/", "filename" => $childDirectoryName, "info" => $directory));
                    else
                        array_push($directoriesArray, array("path" => "$path/" . urlencode($childDirectoryPath) . "/", "filename" => $childDirectoryName, "info" => $directory));
                }
            }
    
            if($numScripts > 0) {
                foreach($dirObj[kSTR_SERVICES] as $serviceName) {
                    $service = $this->home["allServices"][$serviceName];
                    
                        // If the service has been marked as being WAP compatible, display it. Otherwise, skip!
                        
                    if($service["wap_compatible"]["value"]) {
                        $name = $service["name"]["value"];

                        array_push($scriptsArray, array("path" => $serviceName, "service" => $service));
                    
                            // Create the media directory for the service
                        
                        $this->Media_Directory_Generate("service", $serviceName, $service["path_service"]);
                    }
                }
            }
        }       
        
            // Assign the directory and script service arrays to the page template
        
        $result["directories"] = $directoriesArray;
        $result["services"] = $scriptsArray;
        
        return $result;
    }
    
}








?>