<?php

/* Copyright (C) 2003 Adam Tow

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
   


/* Class:       XMLRPC_Client
 *
 * Extends:     Client
 *
 * Description: This client is used for handling XML-RPC requests
 *
 * Revision History:
 *
 *  2003-03-15 - Initial version (adam)
 *
 *
 */

class XMLRPC_Client extends Client
{   
    function XMLRPC_Client()
    {
        $this->Client();
        $this->name = "XMLRPC";
    }
    
    
    
    /* Method:      Main
     *
     * Description: Main routine for the server. We could be called in two different ways:
     *
     *                  1. XML-RPC request from a remote server
     *                  2. Internal request from the web-based client on this server
     *
     * Parameters:  None
     *
     * Returns:     true
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */
     
    function Main()
    {   
        global $_CONF;
        
        $xmlrpc_request = XMLRPC_parse($GLOBALS["HTTP_RAW_POST_DATA"]);     
        $methodName = XMLRPC_getMethodName($xmlrpc_request); 
        $params = XMLRPC_getParams($xmlrpc_request); 
        $paramsLen = sizeof($params);
        
            // If XML-RPC access is turned off, we only let the serialize and serializeError
            // functions through.
            
        if( !$_CONF["allow_xmlrpc_access"] &&
            ($methodName != "soybo.serialize") && $methodName != "soybo.serializeError") {
        
            $this->Error("100", "XML-RPC is not allowed on this Soybo server.");
            
        } else {
            switch($methodName)
            {
                case "soybo.login":             $this->Login($params[0]);
                                                break;
                
                case "soybo.serialize":         $this->SerializeResponse($params[0]);
                                                break;
                
                case "soybo.serializeError":    $this->SerializeError($params[0]);
                                                break;
                                        
                default:                        $this->Run($methodName, $params[0], $params[1]);
                                                break;
            }
        }
        
        return true;
    }   
    

    
    /* Method:      Login
     *
     * Description: This function attempts to the log the user into the system. On success,
     *              we return the sessionID so the user can perform subsequent calls
     *              without having to log in again.
     *
     * Parameters:  authenticationRequest - array containing authentication information
     *
     * Returns:     response packet with the sessionID or an error response
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     *
     */
     
    function Login($authenticationRequest)
    {
        global $_CONF;
        
        $_CONF["username"] = $authenticationRequest["username"];
        
        list($success, $response) = $this->server->Login($authenticationRequest);
        
        if($success) {
            $username = $authenticationRequest["username"];
            
            
            $home = $response;
            
            
            
            $result = $this->workspace->Retrieve($username);
            
            $this->workspace->Set("home", $response);
            
            if($result) {
                $this->currPath = $this->workspace->Get("path");
                $this->currScript = $this->workspace->Get("script");
                $_CONF["locale"] = GFn_Get_Locale($this->workspace->Get("locale"));
            } else {
                $this->current_directory = "";
                $this->current_service = "";
                $this->workspace->Set("username", $username);
                $this->workspace->Set("locale", $_CONF["locale_name"]);
                $this->workspace->Set("page", false);
            }
            $this->workspace->Save();
            $this->home = $response;
        
                // Clean up the home object for XML-RPC Clients
                // We just send a list of the services that are available for the
                // user, getting rid of the directory references, etc.
            
            $methods = array();
            
            foreach($this->home["services"] as $service) {
                $methodName = "soybo." . $service["service_name"];
                $methods[$methodName] = array("name" => $service["name"], "description" => $service["description"]);
            }
            
            XMLRPC_response(XMLRPC_prepare(
                array(  "session_id" => $_SESSION["session_id"],
                        "version" => "1.0",
                        "services" => $methods,
                        
                        //"workspace" => array(
                          //                      "home" => $this->home,
                                               // "locale" => $this->workspace->Get("locale"),
                                               // "path" => $this->workspace->Get("path"),
                                               // "page" => $this->workspace->Get("page"),
                                               // "username" => $this->workspace->Get("username"),
                                               // "script" => $this->workspace->Get("script"),
                                            //))));
                                            )));
            
            
          //  XMLRPC_response(XMLRPC_prepare(array("session_id" => $_SESSION["session_id"])));
        
        } else {
            $faultCode = $result[1]["faultCode"];
            $faultString = $result[1]["faultString"];
            
            return $this->Error($faultCode, $faultString);
        }
    }


    /* Method:      Run
     * 
     * Description: If we are accessing the server purely through XML-RPC calls
     *              we don't have any way of saving session information between calls.
     *              This method exists for people to run their scripts in ways other
     *              than using a web-based client (i.e. an XML-RPC calling application).
     *
     * Parameters:  methodName - the method to run in the service's script
     *              authenticationFrame - frame containing auth, username, password, etc.
     *              parameters - script and parameters to run
     *
     * Returns:     the XML-RPC response or error from running the script
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     */
     
    function Run($methodName, $authenticationFrame, $parameters)
    {   
        global $_CONF;
        global $_DATABASE;
        
        $locale = $_CONF["locale"]->GetStrings();
        
            // Check to see if the first part of the methodName is "soybo."
            
        if(strpos($methodName, "soybo.") === false)
            $this->Error("", $locale["errMethodName"] . $methodName);
        
            // Authenticate the user    
        
        $session_id = $authenticationFrame["session_id"];
        $result = $this->server->Session_Valid($session_id);
                
        switch($result) {
            case kSESSION_VALID:            break;

            case kSESSION_EXPIRED:      $this->Error("", $locale["errSessionExpired"]);
                                        return;
                                        break;
            
            case kSESSION_INVALID:      $this->Error("", $locale["errSessionInvalid"]);
                                        break;
            case kSESSION_NONE:     
            default:                    $this->Error("", $locale["errLoginRequired"]);
                                        return;
                                        break;
        }
        
        $username = $_DATABASE->Get_Username_From_Session($session_id);
                
        if($username) {
            $_CONF["username"] = $username;
            
                // Get the user's workspace
            
            $this->workspace->Retrieve($username);
        
                // Now we get the script that we want to run from the $methodName parameter

            $home = $this->workspace->Get("home");
            
            $methodParts = explode(".", $methodName);
            $service_name = $methodParts[1];
            $methodName = ($methodParts[2]) ? $methodParts[2] : "";
            $service = $home["services"][$service_name];
            
                // Make sure that the service exists and that the user
                // has permission to run the service
                        
            if(!$service || $err = $this->server->Verify_Service($username, $service_name))
               return $this->Error("404", $locale["errServiceNotFound"] . $serviceName);
           
            $path_service = $_CONF["path_services"] . $service["path"] . "/";
            $path_script = $path_service . $service["script"];
            
            $this->Generate_Service_Media($service_name, $path_service);
            
                // Retrieve the service's locale if necessary
            
            if(!$service["locale"]) {
                $service["locale"] = $this->server->Get_Service_Locale($path_service, $service["default_locale"]);
            }
            $params["locale"] = $service["locale"];
            
                // Fill in parameters that will be sent to the script
                
            $params["PHPSESSID"] = $authenticationFrame["session_id"];
            $params["path_media"] = $this->Media_Path_Generate($service_name, true) . "/";
            $params["path_service"] = $path_service;
            
                // Run the script
                
            list($success, $response) = $this->server->Run($path_script, $methodName, $params);

            if($success) {
        
                    // Log the event
            
                if($methodName) GFn_Log_Event(kLOG_USER, "$serviceName.$methodName");
                else GFn_Log_Event(kLOG_USER, "$serviceName");
        
                $xmlrpc_request = XMLRPC_parse($response);
                
                if(isset($xmlrpc_request['methodResponse']['fault'])){
                    $error = XMLRPC_adjustValue(&$xmlrpc_request['methodResponse']['fault']['value']);
                    
                    if($error["faultcode"])
                        $faultCode = $error["faultcode"];   // Handle stupid AppleScript which changes variables to lowercase
                    else $faultCode = $error["faultCode"];
                    
                    if($error["faultstring"])
                        $faultString = $error["faultstring"];   // Handle stupid AppleScript which changes variables to lowercase
                    else $faultString = $error["faultString"];
                    
                    $this->Error($faultCode, $faultString);

                    return;
                } else {
                    $response = XMLRPC_adjustValue(&$xmlrpc_request['methodResponse']['params']['param']['value']);
                    
                    return XMLRPC_response(XMLRPC_prepare($response));
                }
                
            } else {
                $this->Error("", $locale["errRun"]);
            }
        } else {
            GFn_Log_Event(kLOG_ERROR, "User could not be found in database with given Session ID!");

            $this->Error("", $locale["errUnableToProcess"]);
        }
    }



    /* Method:      Error
     * 
     * Description: Returns an error XML-RPC response to the XML-RPC client
     *
     * Parameters:  faultCode - the fault code to display
     *              faultString - the fault string to display
     *
     * Returns:     the XML-RPC error
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     */
     
    function Error($faultCode, $faultString)
    {       
        XMLRPC_error($faultCode, $faultString, WEBLOG_XMLRPC_USERAGENT);
    }
    


    /* Method:      SerializeResponse
     * 
     * Description: This method is used by AppleScript service scripts to transform
     *              AppleScript native data structures into XML-RPC data structures.
     *
     * Parameters:  params - the parameters to serialize
     *
     * Returns:     the serialized parameters
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     */

    function SerializeResponse($params)
    {
        $data["methodResponse"]["params"]["param"]["value"] = XMLRPC_prepare($params);
        
        XMLRPC_response(XMLRPC_prepare(XML_serialize($data)));
    }



    /* Method:      SerializeError
     * 
     * Description: This method is used by AppleScript service scripts to transform
     *              an error object into an XML-RPC error data structure.
     *
     * Parameters:  params - the parameters to serialize
     *
     * Returns:     the serialized parameters
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     */
    
    function SerializeError($params)
    {
        $data["methodResponse"]["fault"]["value"] = XMLRPC_prepare($params);
        
        XMLRPC_response(XMLRPC_prepare(XML_serialize(($data))));
    }
}
    
?>
