<?php

/* Copyright (C) 2003 Adam Tow

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
   

/* Class:       PHP_Script_Handler
 *
 * Description: Script handler for PHP service scripts
 *
 * Revision History:
 *
 *  2003-03-15 - Initial version (adam)
 *
 *
 */

class PHP_Script_Handler extends Script_Handler
{
    var $name;
    
    function PHP_Script_Handler()
    {
        $this->Script_Handler();
        $this->name = "PHP Script Handler";
    }



    /* Method:      Execute
     *
     * Description: Includes the script and run it
     *
     * Parameters:  scriptPath - the path to the script
     *              methodName - the method to call in the script
     *              paramters - parameters to send to the script
     *
     * Returns:     the result of the script
     *
     * Revision History:
     *
     *  2003-03-15 - Initial version (adam)
     *
     */
     
    function RunScript($scriptPath, $methodName, $parameters)
    {   
        include($scriptPath);

        return RunWithParams($methodName, $parameters);
    }

}



/* Function:    ScriptResponse
 *
 * Description: Scripts call this function when they want to return a non-error
 *              response back to the server. We take the PHP return object and
 *              convert it into an XML-RPC response document.
 *
 * Parameters:  response - the PHP object to encode into an XML-RPC document
 *
 * Returns:     serialized response
 *
 * Revision History:
 *
 *  2003-03-15 - Initial version (adam)
 *
 */
     
function ScriptResponse($response)
{
    $data["methodResponse"]["params"]["param"]["value"] = XMLRPC_prepare($response);
    
    return XML_serialize($data);
}



/* Function:    ScriptErrorResponse
 *
 * Description: This function is called when the script wants to return an error
 *              to the user. We encode the faultCode and faultString into a
 *              standard XML-RPC error response document.
 *
 * Parameters:  faultCode - the faultCode to encode
 *              faultString - the faultString of the error
 *
 * Returns:     serialized response
 *
 * Revision History:
 *
 *  2003-03-15 - Initial version (adam)
 *
 */

function ScriptErrorResponse($faultCode, $faultString)
{
    $data["methodResponse"]["fault"]["value"] = XMLRPC_prepare(array("faultCode" => $faultCode, "faultString" => $faultString));
    
    return XML_serialize($data);
}

?>